import copy
import functools


def deepcopy_wrapper(func):
    @functools.wraps(func)
    def wrapper(ol,*args,**kwargs):
        deepcopy = kwargs['deepcopy'] if('deepcopy' in kwargs) else True
        nl = copy.deepcopy(ol) if(deepcopy) else ol
        rslt = func(nl,*args,**kwargs)
        return(rslt)
    return(wrapper)

def keep_ptr_replace(ol,nl):
    ol.clear()
    ol.extend(nl)
    return(ol)


def inplace_wrapper(func):
    @functools.wraps(func)
    def wrapper(ol,*args,**kwargs):
        deepcopy = kwargs['deepcopy'] if('deepcopy' in kwargs) else True
        nl = copy.deepcopy(ol)
        nl = func(nl,*args,**kwargs)
        rslt = nl if(deepcopy) else keep_ptr_replace(ol,nl)
        return(rslt)
    return(wrapper)


def identity(obj):
    return(obj)



def dflt_kwargs(k,dflt,**kwargs):
    if(k in kwargs):
        v = kwargs[k]
    else:
        v = dflt
    return(v)



class _Undefined():
    def __repr__(self):
        return('undefined')

class _Null():
    def __repr__(self):
        return(null)



undefined = _Undefined()
null = _Null()




from .cmmn import inplace_wrapper

@inplace_wrapper
def mapfivo(ol,map_funcs,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        map_func = map_funcs[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(index,value,*other_args)
        rslt.append(ele)
    return(rslt)    


@inplace_wrapper
def mapfiv(ol,map_funcs,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        map_func = map_funcs[i]
        ele = map_func(index,value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfio(ol,map_funcs,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        map_func = map_funcs[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(index,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfvo(ol,map_funcs,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        map_func = map_funcs[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapivo(ol,map_func,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    map_func = map_func
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(index,value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfio(ol,map_funcs,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        map_func = map_funcs[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(index,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfi(ol,map_funcs,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        map_func = map_funcs[i]
        ele = map_func(index,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfv(ol,map_funcs,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        map_func = map_funcs[i]
        ele = map_func(value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapfo(ol,map_funcs,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        map_func = map_funcs[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapiv(ol,map_func,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        ele = map_func(index,value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapio(ol,map_func,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        other_args = map_func_other_args_array[i]
        ele = map_func(index,*other_args)
        rslt.append(ele)
    return(rslt)

@inplace_wrapper
def mapvo(ol,map_func,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        other_args = map_func_other_args_array[i]
        ele = map_func(value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapf(ol,map_funcs,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        map_func = map_funcs[i]
        ele = map_func(*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapi(ol,map_func,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        ele = map_func(index,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapv(ol,map_func,*other_args,**kwargs):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        ele = map_func(value,*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapo(ol,map_func,map_func_other_args_array,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        other_args = map_func_other_args_array[i]
        ele = map_func(*other_args)
        rslt.append(ele)
    return(rslt)


@inplace_wrapper
def mapiv_with_dual(ol,map_func,index_map_func,map_func_other_args,index_map_func_other_args,**kwargs):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = index_map_func(i,*index_map_func_other_args)
        value = map_func(index,ol[i],*_map_func_other_args)
        rslt.append(value)
    return(rslt)


def for_eachfivo(ol,for_each_funcs,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = i
        value = ol[i]
        for_each_func = for_each_funcs[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(index,value,*other_args)
        ol[i] = value
    return(ol)

def for_eachfiv(ol,for_each_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        index = i
        value = ol[i]
        for_each_func = for_each_funcs[i]
        value = for_each_func(index,value,*other_args)
        ol[i] = value
    return(ol)

def for_eachfio(ol,for_each_funcs,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = i
        for_each_func = for_each_funcs[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(index,*other_args)
        ol[i] = value
    return(ol)

def for_eachfvo(ol,for_each_funcs,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        value = ol[i]
        for_each_func = for_each_funcs[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(value,*other_args)
        ol[i] = value
    return(ol)

def for_eachivo(ol,for_each_func,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = i
        value = ol[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(index,value,*other_args)
        ol[i] = value
    return(ol)

def for_eachfio(ol,for_each_funcs,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = i
        for_each_func = for_each_funcs[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(index,*other_args)
        ol[i] = value
    return(ol)

def for_eachfi(ol,for_each_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        index = i
        for_each_func = for_each_funcs[i]
        value = for_each_func(index,*other_args)
        ol[i] = value
    return(ol)

def for_eachfv(ol,for_each_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        value = ol[i]
        for_each_func = for_each_funcs[i]
        value = for_each_func(value,*other_args)
        ol[i] = value
    return(ol)

def for_eachfo(ol,for_each_funcs,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        for_each_func = for_each_funcs[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(*other_args)
        ol[i] = value
    return(ol)

def for_eachiv(ol,for_each_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        index = i
        value = ol[i]
        value = for_each_func(index,value,*other_args)
        ol[i] = value
    return(ol)

def for_eachio(ol,for_each_func,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = i
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(index,*other_args)
        ol[i] = value
    return(ol)

def for_eachvo(ol,for_each_func,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        value = ol[i]
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(value,*other_args)
        ol[i] = value
    return(ol)

def for_eachf(ol,for_each_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        for_each_func = for_each_funcs[i]
        value = for_each_func(*other_args)
        ol[i] = value
    return(ol)

def for_eachi(ol,for_each_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        index = i
        value = for_each_func(index,*other_args)
        ol[i] = value
    return(ol)

def for_eachv(ol,for_each_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    for i in range(0,lngth):
        value = ol[i]
        value = for_each_func(value,*other_args)
        ol[i] = value
    return(ol)

def for_eacho(ol,for_each_func,for_each_func_other_args_array):
    lngth = ol.__len__()
    for i in range(0,lngth):
        other_args = for_each_func_other_args_array[i]
        value = for_each_func(*other_args)
        ol[i] = value
    return(ol)

def for_eachiv_with_dual(ol,for_each_func,index_for_each_func,for_each_func_other_args,index_for_each_func_other_args):
    lngth = ol.__len__()
    for i in range(0,lngth):
        index = index_for_each_func(i,*index_for_each_func_other_args)
        value = for_each_func(index,ol[i],*_for_each_func_other_args)
        ol[i] = value
    return(ol)


@inplace_wrapper
def intlize(ol):
    return(list(map(lambda ele:int(ele),ol)))

@inplace_wrapper
def strlize(ol):
    return(list(map(lambda ele:str(ele),ol)))




def el2iteml(el,k):
    iteml = list(map(lambda ele:ele.__getitem__(k),el))
    return(iteml)

def el2attrl(el,attr):
    attrl = list(map(lambda ele:ele.__getattribute__(attr),el))
    return(attrl)




def reduce_left(ol,callback,initialValue):
    length = ol.__len__()
    accumulator = initialValue
    for i in range(0,length):
        accumulator = callback(accumulator,ol[i])
    return(accumulator)


def reduce_right(ol,callback,initialValue):
    length = ol.__len__()
    accumulator = initialValue
    for i in range(length-1,-1,-1):
        accumulator = callback(accumulator,ol[i])
    return(accumulator)





from .map import mapv

def get_via_pl(ol,pathlist):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        get_via_pl(y,[1,1])
    '''
    this = ol
    for i in range(0,pathlist.__len__()):
        key = pathlist[i]
        this = this.__getitem__(key)
    return(this)

def get_via_pl2(pathlist,ol):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        get_via_pl2([1,1],y)
    '''
    this = ol
    for i in range(0,pathlist.__len__()):
        key = pathlist[i]
        this = this.__getitem__(key)
    return(this)


def get_via_sibseqs(ol,*sibseqs):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        get_via_sibseqs(y,1,1)
    '''
    pathlist = list(sibseqs)
    this = ol
    for i in range(0,pathlist.__len__()):
        key = pathlist[i]
        this = this.__getitem__(key)
    return(this)


def set_via_pl(ol,pathlist,value):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        setitem_via_pathlist(y,"500",[1,1])
        y
    '''
    this = ol
    for i in range(0,pathlist.__len__()-1):
        key = pathlist[i]
        this = this.__getitem__(key)
    this.__setitem__(pathlist[-1],value)
    return(ol)

def set_via_sibseqs(ol,value,*sibseqs):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        setitem_via_sibseqs(y,"500",1,1)
        y
    '''
    this = ol
    pathlist = list(sibseqs)
    for i in range(0,pathlist.__len__()-1):
        key = pathlist[i]
        this = this.__getitem__(key)
    this.__setitem__(pathlist[-1],value)
    return(ol)


def del_via_pl(ol,pathlist):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        delitem_via_pathlist(y,[1,1])
        y
    '''
    this = ol
    for i in range(0,pathlist.__len__()-1):
        key = pathlist[i]
        this = this.__getitem__(key)
    this.__delitem__(pathlist[-1])
    return(ol)


def del_via_sibseqs(ol,*sibseqs):
    '''
        from elist.elist import *
        y = ['a',['b',["bb"]],'c']
        y[1][1]
        delitem_via_sibseqs(y,1,1)
        y
    '''
    pathlist = list(sibseqs)
    this = ol
    for i in range(0,pathlist.__len__()-1):
        key = pathlist[i]
        this = this.__getitem__(key)
    this.__delitem__(pathlist[-1])
    return(ol)




def set_via_il_vl(ol,indexes,values):
    for i in range(indexes.__len__()):
        ol[i] = values[i]
    return(ol)

def set_via_ivlist(ol,*iv_tuples):
    iv_tuples = list(iv_tuples)
    for t in iv_tuples:
        ol[t[0]] = t[1]
    return(ol)


def pl_to_bracket_str(path_list):
    '''
        >>> pl_to_bracket_str([1, '1', 2])
            "[1]['1'][2]"
        >>>
    '''
    t1 = path_list.__repr__()
    t1 = t1.lstrip('[')
    t1 = t1.rstrip(']')
    t2 = t1.split(", ")
    s = ''
    for i in range(0,t2.__len__()):
        s = ''.join((s,'[',t2[i],']'))
    return(s)



def bracket_str_to_pl(gs):
    '''
        gs = "[1]['1'][2]"
        bracket_str_to_pl(gs)
        gs = "['u']['u1']"
        bracket_str_to_pl(gs)
    '''
    def numize(w):
        try:
            int(w)
        except:
            try:
                float(w)
            except:
                return(w)
            else:
                return(float(w))
        else:
           return(int(w))
    def strip_quote(w):
        if(type(w) == type('')):
            if(w[0]==w[-1]):
                if((w[0]=="'") |(w[0]=='"')):
                    return(w[1:-1])
                else:
                    return(w)
            else:
                return(w)
        else:
            return(w)
    gs = gs[1:-1]
    pl = gs.split("][")
    pl = mapv(pl,numize)
    pl = mapv(pl,strip_quote)
    return(pl)






from .map import mapv


def _dflt_vgroup_cond_func(v,*o):
    return((v == o[0]))


def group_indexes_via_value(ol,*args,**kwargs):
    cond_func = _dflt_vgroup_cond_func if(not("cond_func" in kwargs)) else kwargs["cond_func"]
    rslt = []
    length = ol.__len__()
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = cond_func(ol[cursor],*args)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
        elif((not(cond1)) & (not(cond2))):
            rslt.append(slice)
            begin = None
            slice = []
        else:
            pass
        cursor = cursor + 1
    if(slice):
        rslt.append(slice)
    else:
        pass
    return(rslt)


def get_vgroupi_brkseqs(ol,*args,**kwargs):
    '''
        >>> vgroupi_brkseqs([0, 0, 0, 1, 2, 0, 2],0)
        [3, 6]
    '''
    slc = group_indexes_via_value(ol,*args,**kwargs)
    brkseqs = list(map(lambda ele:ele[-1]+1,slc))
    return(brkseqs)




def refl_brkseqs(refl):
    '''
        >>> refl_brkseqs([0, 0, 0, 1, 2, 2, 2])
        [3, 4]
    '''
    brks = []
    prev_value = refl[0]
    for i in range(1,len(refl)):
        curr_value = refl[i]
        if(curr_value == prev_value):
            pass
        else:
            brks.append(i)
            prev_value = curr_value
    return(brks)



def refl_vgroupv(refl):
    '''
        >>> refl_vgroupv([0, 0, 0, 1, 2, 2, 2])
        [[0, 0, 0], [1], [2, 2, 2]]
    '''
    rslts = []
    prev_value = refl[0]
    prev_index = 0
    for i in range(1,len(refl)):
        curr_value = refl[i]
        if(curr_value == prev_value):
            pass
        else:
            slc = refl[prev_index:i]
            rslts.append(slc)
            prev_value = curr_value
            prev_index = i
    slc = refl[prev_index:]
    rslts.append(slc)
    return(rslts)



def refl_vgroupi(refl):
    '''
        >>> refl_vgroupi([0, 0, 0, 1, 2, 2, 2])
        [[0, 1, 2], [3], [4, 5, 6]]
    '''
    rslts = []
    prev_value = refl[0]
    prev_index = 0
    lngth = len(refl)
    for i in range(1,lngth):
        curr_value = refl[i]
        if(curr_value == prev_value):
            pass
        else:
            slc = list(range(prev_index,i))
            rslts.append(slc)
            prev_value = curr_value
            prev_index = i
    slc = list(range(prev_index,lngth))
    rslts.append(slc)
    return(rslts)



def group_by_refl(ol,refl):
    '''

    '''
    rslts = []
    prev_value = refl[0]
    prev_index = 0
    for i in range(1,len(refl)):
        curr_value = refl[i]
        if(curr_value == prev_value):
            pass
        else:
            slc = ol[prev_index:i]
            rslts.append(slc)
            prev_value = curr_value
            prev_index = i
    slc = ol[prev_index:]
    rslts.append(slc)
    return(rslts)


def group_by_lngth(l):
    st = set({})
    rslt = {}
    for i in range(len(l)):
        lngth = len(l[i])
        if(lngth in st):
            rslt[lngth].append(l[i])
        else:
            st.add(lngth)
            rslt[lngth] = [l[i]]
    return(rslt)



def group_by_attr_lngth(l,attrname):
    st = set({})
    rslt = {}
    for i in range(len(l)):
        a = l[i].__getattribute__(attrname)
        lngth = len(a)
        if(lngth in st):
            rslt[lngth].append(l[i])
        else:
            st.add(lngth)
            rslt[lngth] = [l[i]]
    return(rslt)




def group_by_value_lngth(l,keyname):
    st = set({})
    rslt = {}
    for i in range(len(l)):
        v = l[i].__getitem__(keyname)
        lngth = len(v)
        if(lngth in st):
            rslt[lngth].append(l[i])
        else:
            st.add(lngth)
            rslt[lngth] = [l[i]]
    return(rslt)



def group_by_head_str(arr,n):
    ndict = {}
    for i in range(len(arr)):
        s = arr[i]
        s = s[0:n]
        cond = (s in ndict)
        if(cond):
            ndict[s].append(arr[i])
        else:
            ndict[s] = [arr[i]]
    return(ndict)


def group_values_using_continuous_same(l):
    '''
        l = [1,"a","a",2,2,"a",1,"a","b","a",5,"b","b","b"]
        >>> groupv_via_same(l)
        [[1], ['a', 'a'], [2, 2], ['a'], [1], ['a'], ['b'], ['a'], [5], ['b', 'b', 'b']]
        >>>
    '''
    rslt = []
    lngth = len(l)
    if(lngth ==0):
        return([])
    else:
        cursor = 0
        value = l[0]
        cache = []
        while(cursor < lngth):
            cond = (l[cursor] == value)
            if(cond):
                cache.append(value)
            else:
                rslt.append(cache)
                value = l[cursor]
                cache = [value]
            cursor = cursor + 1
        if(len(cache)>0):
            rslt.append(cache)
    return(rslt)



def group_indexes_using_continuous_same(l):
    '''
        l = [1,"a","a",2,2,"a",1,"a","b","a",5,"b","b","b"]
        >>> groupi_via_same(l)
        [[0], [1, 2], [3, 4], [5], [6], [7], [8], [9], [10], [11, 12, 13]]
        >>>
    '''
    rslt = []
    lngth = len(l)
    if(lngth ==0):
        return([])
    else:
        cursor = 0
        value = l[0]
        cache = []
        while(cursor < lngth):
            cond = (l[cursor] == value)
            if(cond):
                cache.append(cursor)
            else:
                rslt.append(cache)
                value = l[cursor]
                cache = [cursor]
            cursor = cursor + 1
        if(len(cache)>0):
            rslt.append(cache)
    return(rslt)


def group_ranges_via_continuous_same(l):
    '''
        l = [1,"a","a",2,2,"a",1,"a","b","a",5,"b","b","b"]
        >>> group_range_via_same(l)
        [[0, 1], [1, 3], [3, 5], [5, 6], [6, 7], [7, 8], [8, 9], [9, 10], [10, 11], [11, 14]]
        >>>
    '''
    def map_func(ele):
        if(len(ele)==1):
            ele = [ele[0],ele[0]+1]
        else:
            ele = [ele[0],ele[-1]+1]
        return(ele)
    arr = group_indexes_using_continuous_same(l)
    rngs = mapv(arr,map_func)
    return(rngs)




from .map   import mapv
from .init  import from_range 
from .cmmn  import inplace_wrapper
from .index import uniform_index,indexes_all
from .sort  import sort_refer_to
from .fltr  import fltrv
from .util  import uniqualize
from .crud  import append,prepend

def compress(ol):
    '''
        #only support sorted-ints or sorted-ascii
        l = [1,5,6,7,8,13,14,18,30,31,32,33,34]
        compress(l)
        l = [1,5,6,7,8,13,14,18,30,31,32,33,34,40]
        compress(l)
        l = ['a','b','c','d','j','k','l','m','n','u','y','z']
        compress(l)
    '''
    T = (type(ol[0]) == type(0))
    if(T):
        l = ol
    else:
        l = mapv(ol,ord)
    length = l.__len__()
    secs = []
    si = 0
    ei = 0
    prev = l[0]
    for i in range(1,length):
        curr = l[i]
        cond = (curr == (prev+1))
        if(cond):
            ei = i
            prev = curr
        else:
            if(T):
                sec = (l[si],l[ei])
            else:
                sec = (ol[si],ol[ei])
            if(si == ei):
                sec = sec[0]
            else:
                pass
            secs.append(sec)
            si = i
            ei = i
            prev = curr
    if(T):
        sec = (l[si],l[ei])
    else:
        sec = (ol[si],ol[ei])
    if(si == ei):
        sec = sec[0]
    else:
        pass
    secs.append(sec)
    return(secs)




def decompress(cl):
    '''
        #only support sorted-ints or sorted-ascii
        cl = [1, (5, 8), (13, 14), 18, (30, 34)]
        decompress(cl)
        cl = [1, (5, 8), (13, 14), 18, (30, 34), 40]
        decompress(cl)
        cl = [('a', 'd'), ('j', 'n'), 'u', ('y', 'z')]
        decompress(cl)
    '''
    def cond_func(ele):
        length = ele.__len__()
        cond = (length == 1)
        if(cond):
            return(ord(ele))
        else:
            x = ord(ele[0])
            y = ord(ele[1])
            return((x,y))
    if(type(cl[0])==type(0)):
        T = True
    elif(cl[0].__len__() == 1):
        T = (type(cl[0]) == type(0))
    else:
        T = (type(cl[0][0]) == type(0))
    if(T):
        l = cl
    else:
        l = mapv(cl,cond_func)
    rslt = []
    for i in range(0,l.__len__()):
        ele = l[i]
        if(type(ele) == type(0)):
            arr = [ele]
        elif(ele.__len__() == 1):
            arr = [ele]
        else:
            sv = ele[0]
            ev = ele[1]
            arr = from_range(sv,ev+1,1)
        if(T):
            pass
        else:
            arr = mapv(arr,chr)
        rslt.extend(arr)
    return(rslt)





def spanize(break_points,length):
    bps = mapv(break_points,uniform_index,length)
    bps.sort()
    bps = prepend(bps,0)
    bps = append(bps,length)
    bps = uniqualize(bps)
    bpslen = bps.__len__()
    secs=[(0,bps[0])]
    for i in range(0,bpslen-1):
        r = (bps[i],bps[i+1])
        secs.append(r)
    secs.append((bps[bpslen-1],length))
    if(secs[0][0] == secs[0][1]):
        secs.pop(0)
    else:
        pass
    if(secs[-1][0] == secs[-1][1]):
        secs.pop(-1)
    else:
        pass
    return(secs)


def fullfill_spans(spans,lngth):
    brkpnts = []
    for i in range(0,spans.__len__()):
        span = spans[i]
        brkpnts.append(span[0])
        brkpnts.append(span[1])
    return(spanize(brkpnts,lngth))


def get_supplement_of_spans(spans,lngth):
    rslt = []
    si = 0
    ei = spans[0][0]
    if(si == ei):
        pass
    else:
        rslt.append((si,ei))
    prev_ei = spans[0][1]
    for i in range(1,spans.__len__()):
        si = prev_ei
        ei = spans[i][0]
        rslt.append((si,ei))
        prev_ei = spans[i][1]
    if(prev_ei < lngth):
        rslt.append((prev_ei,lngth))
    else:
        rslt.append((prev_ei,lngth+1))
    rslt = fltrv(rslt,lambda r:r[0]!=r[1])
    return(rslt)


def get_span_loc(spans,word_loc):
    for i in range(0,spans.__len__()):
        span = spans[i]
        if((word_loc>=span[0])&(word_loc<span[1])):
            return(i)
        else:
            pass
    return(None)




def broke_via_seqs(ol,break_points):
    bps = list(break_points)
    length = ol.__len__()
    rgs = spanize(bps,length)
    rslt = []
    for i in range(0,rgs.__len__()):
        si,ei = rgs[i]
        sec = ol[si:ei]
        rslt.append(sec)
    return(rslt)



def broke_via_some(ol,*break_points):
    bps = list(break_points)
    return(broke_via_seqs(ol,bps))



def where_index_interval(ol,value):
    si = None
    ei = None
    for i in range(0,ol.__len__()):
        ele = ol[i]
        if(value >ele):
            si = i
        elif(value == ele):
            return((i,i))
        else:
            ei = i
            return((si,ei))
    return((si,ei))



def where_value_interval(ol,value):
    '''
        ol = [0, 4, 6, 8, 10, 14]
        value_interval(ol,-1)
        value_interval(ol,1)
        value_interval(ol,2)
        value_interval(ol,3)
        value_interval(ol,4)
        value_interval(ol,9)
        value_interval(ol,14)
        value_interval(ol,17)
    '''
    si,ei = where_index_interval(ol,value)
    if(si == None):
        sv = None
    else:
        sv = ol[si]
    if(ei == None):
        ev = None
    else:
        ev = ol[ei]
    return((sv,ev))


def upper_bound(ol,value):
    return(where_value_interval(ol,value)[1])

def lower_bound(ol,value):
    return(where_value_interval(ol,value)[0])



def chunk(ol,interval):
    length = ol.__len__()
    seqs = from_range(0,length,interval)
    rslt = broke_via_seqs(ol,seqs)
    return(rslt)




def comprise(list1,list2,**kwargs):
    if('mode' in kwargs):
        mode = kwargs['mode']
    else:
        mode = "loose"
    len_1 = list1.__len__()
    len_2 = list2.__len__()
    if(len_2>len_1):
        return(False)
    else:
        if(mode=="strict"):
            if(list2 == list1[:len_2]):
                return(True)
            else:
                return(False)
        else:
            end = len_1 - len_2
            for i in range(0,end+1):
                if(list2 == list1[i:(i+len_2)]):
                    return(True)
                else:
                    pass
    return(False)




def diff_indexes(l1,l2):
    '''
        from elist.elist import *
        l1 = [1,2,3,5]
        l2 = [0,2,3,4]
        diff_indexes(l1,l2)
    '''
    rslt = []
    for i in range(0,l1.__len__()):
        if(l1[i]!=l2[i]):
            rslt.append(i)
    return(rslt)

def diff_values(l1,l2):
    '''
        from elist.elist import *
        l1 = [1,2,3,5]
        l2 = [0,2,3,4]
        diff_values(l1,l2)
    '''
    rslt = []
    for i in range(0,l1.__len__()):
        if(l1[i]!=l2[i]):
            rslt.append(l1[i])
    return(rslt)


def same_indexes(l1,l2):
    '''
        from elist.elist import *
        l1 = [1,2,3,5]
        l2 = [0,2,3,4]
        same_indexes(l1,l2)
    '''
    rslt = []
    for i in range(0,l1.__len__()):
        if(l1[i]==l2[i]):
            rslt.append(i)
    return(rslt)

def same_values(l1,l2):
    '''
        from elist.elist import *
        l1 = [1,2,3,5]
        l2 = [0,2,3,4]
        same_values(l1,l2)
    '''
    rslt = []
    for i in range(0,l1.__len__()):
        if(l1[i]==l2[i]):
            rslt.append(l1[i])
    return(rslt)

def intersection(ol1,ol2):
    nl = list(set(ol1).intersection(set(ol2)))
    return(nl)

def shorter(ol1,ol2):
    lngth1 = len(ol1)
    lngth2 = len(ol2)
    if(lngth1 < lngth2):
        return(ol1)
    else:
        return(ol2)

def longer(ol1,ol2):
    lngth1 = len(ol1)
    lngth2 = len(ol2)
    if(lngth1 > lngth2):
        return(ol1)
    else:
        return(ol2)



def ordered_intersection(ol1,ol2):
    nl = []
    ol = shorter(ol1,ol2)
    for i in range(len(ol)):
        cond = (ol1[i] == ol2[i])
        if(cond):
            nl.append(ol1[i])
        else:
            pass
    return(nl)


def fullinfo_intersection(ol1,ol2,**kwargs):
    if("detailed" in kwargs):
        detailed = kwargs['detailed']
    else:
        detailed = False
    len1  = ol1
    len2  = ol2
    lngth = min(len1,len2)
    rslt = []
    for i in range(lngth):
        if(ol1[i] == ol2[i]):
            if(detailed):
                rslt.append({
                    "index":i,
                    "value":ol1[i]
                })
            else:
                rslt.append(ol1[i])
        else:
            pass
    return(rslt)


def union(ol1,ol2):
    nl = list(set(ol1).union(set(ol2)))
    return(nl)

def difference(ol1,ol2):
    nl = list(set(ol1).difference(set(ol2)))
    return(nl)

def fst_cmmn_val_index(l0,l1,**kwargs):
    reverse =  kwargs['reverse'] if("reverse" in kwargs) else False
    lngth = min(len(l0),len(l1))
    if(reverse):
        cursor = None
        for i in range(-1,-1-lngth,-1):
            if(l0[i] == l1[i]):
                cursor = i
            else:
                break
        if(cursor == None):
            # the fst from end is-not-eq
            return(None)
        else:
            rslt = (uniform_index(cursor,len(l0)),uniform_index(cursor,len(l1)))
            return(rslt)
    else:
        for i in range(lngth):
            if(l0[i] == l1[i]):
                return((i,i))
            else:
                pass
        return(None)


def fst_cmmn_val(l0,l1,**kwargs):
    rslt = fst_cmmn_val_index(l0,l1,**kwargs)
    rslt = None if(rslt==None) else l0[rslt[0]]
    return(rslt)





import copy
from .cmmn  import inplace_wrapper
from .index import uniform_index,indexes_all,indexes_some
from .sort  import sort_refer_to
from .map   import mapv
from .fltr  import fltrv
from .slct  import some_keeping_order,some
from .init  import from_range,init


@inplace_wrapper
def append(ol,ele,**kwargs):
    ol.append(ele)
    return(ol)


@inplace_wrapper
def push(ol,ele,**kwargs):
    ol.append(ele)
    return(ol)


@inplace_wrapper
def append_some(ol,*eles,**kwargs):
    return(extend(ol,list(eles),**kwargs))


@inplace_wrapper
def prepend(ol,ele,**kwargs):
    new = [ele]
    new.extend(ol)
    return(new)


@inplace_wrapper
def extend(ol,nl,**kwargs):
    ol.extend(nl)
    return(ol)

@inplace_wrapper
def prextend(ol,nl,**kwargs):
    nl.extend(ol)
    return(nl)

@inplace_wrapper
def prepend_some(ol,*eles,**kwargs):
    return(prextend(ol,list(eles),**kwargs))


@inplace_wrapper
def unshift(ol,*eles,**kwargs):
    return(prextend(ol,list(eles),**kwargs))


@inplace_wrapper
def concat(*arrays,**kwargs):
    new = []
    length = arrays.__len__()
    for i in range(0,length):
        array = arrays[i]
        new.extend(array)
    return(new)

@inplace_wrapper
def cons(head_ele,l,**kwargs):
    return(prepend(l,head_ele,**kwargs))


@inplace_wrapper
def insert(ol,start_index,ele,**kwargs):
    ol.insert(start_index,ele)
    return(ol)


@inplace_wrapper
def insert_some(ol,start_index,*eles,**kwargs):
    start_index = kwargs['index']
    length = ol.__len__()
    si = uniform_index(start_index,length)
    new = ol[0:si]
    new.extend(list(eles))
    new.extend(cpol[si:])
    return(new)


def insert_many(ol,eles,locs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "deepcopy"
    eles = copy.deepcopy(eles)
    locs = copy.deepcopy(locs)
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,locs.__len__()):
        if(locs[i]>=length):
            pass
        else:
            locs[i] = uniform_index(locs[i],length)
    tmp = sort_refer_to(eles,locs)
    eles = tmp['list']
    locs = tmp['referer']
    label = eles.__len__()
    si = 0
    ei = 0
    for i in range(0,locs.__len__()):
        if(locs[i]>=length):
            label = i
            break
        else:
            ei = locs[i]
            new.extend(cpol[si:ei])
            new.append(eles[i])
            si = ei
    for i in range(label,locs.__len__()):
        new.append(eles[i])
    new.extend(cpol[ei:])
    if(mode == "deepcopy"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)



def insert_section(ol,sec,loc,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "deepcopy"
    secs = [sec]
    locs = [loc]
    return(insert_sections_many(ol,secs,locs,mode=mode))




def insert_sections_some(ol,*secs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "deepcopy"
    loc = kwargs['index']
    secs = list(secs)
    secs = [concat(*secs)]
    locs = [loc]
    return(insert_sections_many(ol,secs,locs,mode=mode))


def insert_sections_many(ol,secs,locs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "deepcopy"
    secs = copy.deepcopy(secs)
    locs = copy.deepcopy(locs)
    brked = broke_via_seqs(ol,locs)
    seclen = secs.__len__()
    brklen = brked.__len__()
    if(locs[0]==0):
        new = secs[0]
        length = seclen -1
        if(length < brklen):
            for i in range(0,length):
                new.extend(brked[i])
                new.extend(secs[i+1])
            for i in range(length,brklen):
                new.extend(brked[i])
        elif(length == brklen):
            for i in range(0,length):
                new.extend(brked[i])
                new.extend(secs[i+1])
        else:
            for i in range(0,brklen):
                new.extend(brked[i])
                new.extend(secs[i+1])
            for i in range(brklen,length):
                new.extend(secs[i])
    else:
        new = brked[0]
        length = brklen -1
        if(length < seclen):
            for i in range(0,length):
                new.extend(secs[i])
                new.extend(brked[i+1])
            for i in range(length,seclen):
                new.extend(secs[i])
        elif(length == seclen):
            for i in range(0,length):
                new.extend(secs[i])
                new.extend(brked[i+1])
        else:
            for i in range(0,seclen):
                new.extend(secs[i])
                new.extend(brked[i+1])
            for i in range(seclen,length):
                new.extend(brked[i])
    if(mode == "deepcopy"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)


def repeat_every(l,times):
    nl = []
    for i in range(0,l.__len__()):
        for j in range(0,times):
            nl.append(l[i])
    return(nl)



def apadding(l,lngth,val):
    '''
        >>> padding([1,2,3],6,0)
        [1, 2, 3, 0, 0, 0]
        >>>
    '''
    llen = len(l)
    if(llen>=lngth):
        return(l)
    else:
        tail = init(lngth-llen,val)
        return(l+tail)



def prepadding(l,lngth,val):
    '''
        >>> prepadding([1,2,3],6,0)
        [0, 0, 0, 1, 2, 3]
        >>>
    '''
    llen = len(l)
    if(llen>=lngth):
        return(l)
    else:
        hl = init(lngth-llen,val)
        return(hl+l)




#####
#####

def pop(ol,index,**kwargs):
    index = uniform_index(index,ol.__len__())
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    if(mode == "new"):
        new = copy.deepcopy(ol)
        popped = new.pop(index)
        return({'popped':popped,'list':new})
    else:
        popped = ol.pop(index)
        return(popped)

def shift(ol,**kwargs):
    if('mode' in kwargs):
        mode = kwargs['mode']
    else:
        mode = "new"
    length = ol.__len__()
    rslt = pop(ol,0,mode=mode)
    return(rslt)



def cond_pop(ol,index,**kwargs):
    cond_func = kwargs['cond_func']
    cond_func_args = kwargs['cond_func_args']
    index = uniform_index(index,ol.__len__())
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    value = ol[index]
    cond = cond_func(index,value,*cond_func_args)
    if(mode == "new"):
        new = copy.deepcopy(ol)
        if(cond):
            popped = new.pop(index)
        else:
            popped = new
        return({'popped':popped,'list':new})
    else:
        if(cond):
            popped = ol.pop(index)
        else:
            popped = ol
        return(popped)



def pop_range(ol,start_index,end_index,**kwargs):
    length = ol.__len__()
    start_index = uniform_index(start_index,length)
    end_index = uniform_index(end_index,length)
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    if(mode == "new"):
        cpol = copy.deepcopy(ol)
        new = []
        popped = []
        for i in range(0,start_index):
            new.append(cpol[i])
        for i in range(start_index,end_index):
            popped.append(cpol[i])
        for i in range(end_index,length):
            new.append(cpol[i])
        return({'popped':popped,'list':new})
    else:
        tmp = []
        popped = []
        for i in range(0,start_index):
            tmp.append(ol[i])
        for i in range(start_index,end_index):
            popped.append(ol[i])
        for i in range(end_index,length):
            tmp.append(ol[i])
        ol.clear()
        for i in range(0,tmp.__len__()):
            ol.append(tmp[i])
        return(popped)



def pop_some(ol,*indexes,**kwargs):
    length = ol.__len__()
    indexes = list(map(lambda index:uniform_index(index,length),list(indexes)))
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    if(mode == "new"):
        cpol = copy.deepcopy(ol)
        new = []
        popped = []
        for i in range(0,length):
            if(i in indexes):
                popped.append(cpol[i])
            else:
                new.append(cpol[i])
        return({'popped':popped,'list':new})
    else:
        tmp = []
        popped = []
        for i in range(0,length):
            if(i in indexes):
                popped.append(ol[i])
            else:
                tmp.append(ol[i])
        ol.clear()
        for i in range(0,tmp.__len__()):
            ol.append(tmp[i])
        return(popped)


def another_pop_some(l,*seqs):
    seqs = list(seqs)
    seqs = sorted(seqs)
    count = 0
    for i in range(0,seqs.__len__()):
        seq = seqs[i]
        seq = seq -count
        l.pop(seq)
        count = count + 1
    return(l)



#####
#####


def rm_fst(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    if(mode == "new"):
        new = copy.deepcopy(ol)
        new.remove(value)
        return(new)
    else:
        ol.remove(value)
        return(ol)


def rm_fst_not(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    length = ol.__len__()
    if(mode == "new"):
        new = copy.deepcopy(ol)
        for i in range(0,length):
            if(new[i] == value):
                pass
            else:
                new.pop(i)
                return(new)
        return(new)
    else:
        for i in range(0,length):
            if(ol[i] == value):
                pass
            else:
                ol.pop(i)
                return(ol)
        return(ol)


def rm_lst(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    new = copy.deepcopy(ol)
    new.reverse()
    new.remove(value)
    new.reverse()
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)




def rm_lst_not(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    length = ol.__len__()
    if(mode == "new"):
        new = copy.deepcopy(ol)
        for i in range(length-1,-1,-1):
            if(new[i] == value):
                pass
            else:
                new.pop(i)
                return(new)
        return(new)
    else:
        for i in range(length-1,-1,-1):
            if(ol[i] == value):
                pass
            else:
                ol.pop(i)
                return(ol)
        return(ol)


def rm_which(ol,value,which,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    new = copy.deepcopy(ol)
    length = ol.__len__()
    if(mode == "new"):
        l = new
    else:
        l = ol
    seq = -1
    for i in range(0,length):
        if(ol[i]==value):
            seq = seq + 1
            if(seq == which):
                l.pop(i)
                break
            else:
                pass
        else:
            pass
    return(l)



def rm_which_not(ol,value,which,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    new = copy.deepcopy(ol)
    length = ol.__len__()
    if(mode == "new"):
        l = new
    else:
        l = ol
    seq = -1
    for i in range(0,length):
        if(not(ol[i]==value)):
            seq = seq + 1
            if(seq == which):
                l.pop(i)
                break
            else:
                pass
        else:
            pass
    return(l)


def rm_some(ol,value,*seqs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    seqs = list(seqs)
    new = []
    length = ol.__len__()
    seq = -1
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        if(cpol[i]==value):
            seq = seq + 1
            if(seq in seqs):
                pass
            else:
                new.append(cpol[i])
        else:
            new.append(cpol[i])
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)

def rm_some_not(ol,value,*seqs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    seqs = list(seqs)
    new = []
    length = ol.__len__()
    seq = -1
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        if(not(cpol[i]==value)):
            seq = seq + 1
            if(seq in seqs):
                pass
            else:
                new.append(cpol[i])
        else:
            new.append(cpol[i])
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)


def rm_all(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        if(cpol[i]==value):
            pass
        else:
            new.append(cpol[i])
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)



def rm_all_not(ol,value,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        if(cpol[i]==value):
            new.append(cpol[i])
        else:
            pass
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)



def rm_many(ol,values,seqs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    values = copy.deepcopy(values)
    seqs = copy.deepcopy(seqs)
    cursors = [-1] * values.__len__()
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        label = True
        for j in range(0,cursors.__len__()):
            which = seqs[j]
            value = values[j]
            if(cpol[i] == value):
                cursors[j] = cursors[j] + 1
                if(cursors[j] == which):
                    label = False
                    break
                else:
                    pass
            else:
                pass
        if(label):
            new.append(cpol[i])
        else:
            pass
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)



def rm_many_not(ol,values,seqs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    values = copy.deepcopy(values)
    seqs = copy.deepcopy(seqs)
    cursors = [-1] * values.__len__()
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        label = True
        for j in range(0,cursors.__len__()):
            which = seqs[j]
            value = values[j]
            if(not(cpol[i] == value)):
                cursors[j] = cursors[j] + 1
                if(cursors[j] == which):
                    label = False
                    break
                else:
                    pass
            else:
                pass
        if(label):
            new.append(cpol[i])
        else:
            pass
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)



@inplace_wrapper
def reverse(ol,**kwargs):
    ol.reverse()
    return(ol)




def splice(ol,start,deleteCount,*eles,**kwargs):
    if('mode' in kwargs):
        mode = kwargs['mode']
    else:
        mode = "new"
    length = ol.__len__()
    new = copy.deepcopy(ol)
    if(start >= length):
        eles = list(eles)
        new.extend(eles)
    else:
        start = uniform_index(start,length)
        end = start + deleteCount
        tmp = pop_range(new,start,end,mode="new")['list']
        new = insert_some(tmp,*eles,index=start,mode="new")
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)


def interleave(*arrays,**kwargs):
    '''
        arr1 = [1,2,3,4]
        arr2 = ['a','b','c','d']
        arr3 = ['@','#','%','*']
        interleave(arr1,arr2,arr3)
    '''
    anum = arrays.__len__()
    rslt = []
    length = arrays[0].__len__()
    for j in range(0,length):
        for i in range(0,anum):
            array = arrays[i]
            rslt.append(array[j])
    return(rslt)


def deinterleave(ol,gnum):
    '''
        ol = [1, 2, 3, 4, 5, 6, 7, 8, 9]
        deinterleave(ol,3)

    '''
    def test_func(ele,index,interval,which):
        cond= (index % interval == which)
        return(cond)
    rslt = []
    for i in range(0,gnum):
        arr = fltrv(ol,test_func,gnum,i)
        rslt.append(arr)
    return(rslt)


def split(ol,value,*whiches,**kwargs):
    '''
        ol = ['a',1,'a',2,'a',3,'a',4,'a']
        split(ol,'a')
        split(ol,'a',whiches=[0])
        split(ol,'a',whiches=[1])
        split(ol,'a',whiches=[2])
        split(ol,'a',whiches=[0,2])
        ol = [1,'a',2,'a',3,'a',4]
        split(ol,'a')
        split('x=bcdsef=g','=',whiches=[0])

    '''
    if('whiches' in kwargs):
        whiches = kwargs['whiches']
    else:
        whiches = None
    indexes =  indexes_all(ol,value)
    if(whiches == None):
        pass
    else:
        indexes = some(indexes,*whiches)
    rslt = []
    rslt.append(ol[:indexes[0]])
    si = indexes[0]+1
    for i in range(1,indexes.__len__()):
        ei = indexes[i]
        ele = ol[si:ei]
        rslt.append(ele)
        si = ei + 1
    ele = ol[si:ol.__len__()]
    rslt.append(ele)
    return(rslt)



#####
#####

def repl_some(ol,value,*indexes,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    indexes = list(indexes)
    new = []
    length = ol.__len__()
    cpol = copy.deepcopy(ol)
    for i in range(0,length):
        if(i in indexes):
            new.append(value)
        else:
            new.append(cpol[i])
    if(mode == "new"):
        return(new)
    else:
        ol.clear()
        ol.extend(new)
        return(ol)


def repl_some_eq(ol,src_value,dst_value,*seqs,**kwargs):
    if('mode' in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "new"
    indexes = indexes_some(ol,src_value,*list(seqs))
    return(replace_some(ol,dst_value,*indexes,mode=mode))



def repl_fst_eq(ol,src_value,dst_value,**kwargs):
    return(repl_some_eq(ol,src_value,dst_value,0,**kwargs))

def repl_lst_eq(ol,src_value,dst_value,**kwargs):
    return(repl_some_eq(ol,src_value,dst_value,-1,**kwargs))


def repl_which_eq(ol,src_value,dst_value,which,**kwargs):
    return(repl_some_eq(ol,src_value,dst_value,which,**kwargs))


def repl_all_eq(ol,src_value,dst_value,**kwargs):
    for i in range(len(ol)):
        if(ol[i] == src_value):
            ol[i] = dst_value
        else:
            pass
    return(ol)


def cond_repl_some(ol,cond_func,dst_value,*seqs,**kwargs):
    other_args = kwargs['cond_func_other_args'] if('cond_func_other_args' in kwargs) else []
    for i in range(len(ol)):
        cond0 = cond_func(i,ol[i],*other_args)  
        cond1 = (i in list(seqs))
        cond = (cond0 and cond1)
        if(cond):
            ol[i] = dst_value
        else:    
            pass
    return(ol)


def cond_repl_which(ol,cond_func,dst_value,which,**kwargs):
    return(cond_repl_some(ol,cond_func,dst_value,which,**kwargs))



def cond_repl_fst(ol,cond_func,dst_value,**kwargs):
    return(cond_repl_some(ol,cond_func,dst_value,0,**kwargs))


def cond_repl_lst(ol,cond_func,dst_value,**kwargs):
    other_args = kwargs['cond_func_other_args'] if('cond_func_other_args' in kwargs) else []
    lst_index = None
    for i in range(len(ol)-1,-1,-1):
        cond0 = cond_func(i,ol[i],*other_args)
        cond1 = (i in list(seqs))
        cond = (cond0 and cond1)
        if(cond):
            lst_index = i
            break
        else:
            pass
    if(lst_index != None):
        ol[lst_index] = dst_value
    else:
        pass
    return(ol)


def cond_repl_all(ol,cond_func,dst_value,**kwargs):
    other_args = kwargs['cond_func_other_args'] if('cond_func_other_args' in kwargs) else []
    for i in range(len(ol)):
        cond = cond_func(i,ol[i],*other_args)
        if(cond):
            ol[i] = dst_value
        else:
            pass
    return(ol)





from .fltr import fltrv
from .sort import sort_dtb


def fltrv_via_loose_in(arr,k):
    fltred  = fltrv(arr,lambda ele:(ele in k))
    return(fltred)



def is_loose_in(arr,k):
    for each in arr:
        cond = (each in k)
        if(cond):
            return(True)
        else:
            pass
    return(False)



def fltrv_via_loose_contain(arr,k):
    fltred  = fltrv(arr,lambda ele:(k in ele))
    return(fltred)



def is_loose_contain(arr,k):
    for each in arr:
        cond = (k in each)
        if(cond):
            return(True)
        else:
            pass
    return(False)




def fltrv_via_regex_match(arr,regex):
    def test_func(v):
        m = regex.search(ele)
        cond = False if(m==None) else True
        return(cond)
    fltred  = fltrv(arr,test_func)
    return(fltred)


def is_regex_match(arr,regex):
    def test_func(v):
        m = regex.search(ele)
        cond = False if(m==None) else True 
        return(cond)
    for each in arr:
        cond = test_func(each)
        if(cond):
            return(True)
        else:
            pass
    return(False)




def lcstr(s0,s1):
    len0 = len(s0)
    len1 = len(s1)
    slider_len = len1-1 + len0 + len1-1
    label_s0_end = len1-1 + len0 + 1
    label_s0_start = len1
    def get_si1_ei1(len1,label_s0_end,cursor):
        si1 = len1-1-cursor
        si1 = 0 if(si1 <0) else si1
        ei1 = len1 if((cursor+len1)<(label_s0_end)) else (label_s0_end-cursor-1)
        return((si1,ei1))
    def get_si0_ei0(len1,label_s0_start,label_s0_end,cursor):
        si0 = 0 if(cursor <label_s0_start) else (cursor-len1+1)
        ei0 = len0 if((cursor+len1)>=label_s0_end) else ((cursor+1))
        return((si0,ei0))
    def lcstr_for_same_length_array(arr0,arr1,lngth):
        rslt = []
        i = 0
        si = None
        while(i<lngth):
            if(arr0[i] == arr1[i]):
                if(si == None):
                    si = i
                else:
                    pass
            else:
                if(si != None):
                    t = (si,i)
                    rslt.append(t)
                    si = None
                else:
                    pass
            i = i + 1
        return(rslt)
    def get_comm_substr(len0,len1,label_s0_end):
        rslt = []
        si1_begin = 0
        si1_end = len1-1 + len0 + 1
        si0_begin = len1-1
        for cursor in range(si1_begin,si1_end):
            si1,ei1 = get_si1_ei1(len1,label_s0_end,cursor)
            si0,ei0 = get_si0_ei0(len1,label_s0_start,label_s0_end,cursor)
            subs0 = s0[si0:ei0]
            subs1 = s1[si1:ei1]
            lngth = ei0 - si0
            tmp = lcstr_for_same_length_array(subs0,subs1,lngth)
            for each in tmp:
                rsi,rei = each
                cmms = subs0[rsi:rei]
                if(len(cmms)==0):
                    pass
                else:
                    d = {
                        "pos0":(si0+rsi,si0+rei),
                        "pos1":(si1+rsi,si1+rei),
                        "s":cmms,
                        "len":len(cmms)
                    }
                    rslt.append(d)
            rslt = sort_dtb(rslt,cond_keys=['len'])
        return(rslt)
    rslt = get_comm_substr(len0,len1,label_s0_end)
    return(rslt)



def join(ol,separator=","):
    if(ol.__len__() == 0):
        return("")
    else:
        pass
    cond = (type(ol[0])==type(b''))
    if(cond):
        rslt = b''
    else:
        rslt =""
    length = ol.__len__()
    for i in range(0,length-1):
        ele = ol[i]
        if(cond):
            pass
        else:
            ele = str(ele)
        rslt = rslt + ele + separator
    if(cond):
        rslt = rslt + ol[length - 1]
    else:
        rslt = rslt + str(ol[length - 1])
    return(rslt)



def join_with_sp_list(ol,*sps):
    rslt =""
    length = ol.__len__()
    for i in range(0,length-1):
        rslt = rslt + str(ol[i]) + sps[i]
    rslt = rslt + str(ol[length - 1])
    return(rslt)



def htmljoin(ol,sp,**kwargs):
    '''
        ol = [1,2,3,4]
        htmljoin(ol,"option",outer="select")

    '''
    if('outer' in kwargs):
        outer = kwargs['outer']
    else:
        outer = ""
    if(outer):
        head = "<" + outer + ">"
        tail = "</" + outer + ">"
    else:
        head = ""
        tail = ""
    rslt = head
    length = ol.__len__()
    begin = "<" + sp + ">"
    end = "</" + sp + ">"
    for i in range(0,length):
        rslt = rslt + begin + str(ol[i]) + end
    rslt = rslt + tail
    return(rslt)







from .cmmn import inplace_wrapper


@inplace_wrapper
def iswap(arr,i,j,*kwargs):
    tmp = arr[i]
    arr[i] = arr[j]
    arr[j] = tmp
    return(arr)

@inplace_wrapper
def vswap(arr,v1,v2,**kwargs):
    i1 = arr.index(v1)
    i2 = arr.index(v2)
    arr = iswap(arr,i1,i2,**kwargs)
    return(arr)


@inplace_wrapper
def reindex(arr,*nindexes,**kwargs):
    for i in range(nindexes.__len__()):
        arr[nindexes[i]] = arr[i]
    return(arr)






from operator import itemgetter


def some(ol,*indexes):
    if(indexes.__len__()==0):
        rslt = []
    else:
        rslt = itemgetter(*indexes)(ol)
        rslt = list(rslt)
    return(rslt)

def some_keeping_order(ol,*indexes):
    rslt = []
    for i in range(0,indexes.__len__()):
        index = indexes[i]
        ele = ol[index]
        rslt.append(ele)
    return(rslt)


def not_some(ol,*indexes):
    lngth = len(ol)
    all_index_st = set(list(range(0,lngth,1)))
    unslcted_index_st = set(indexes)
    slcted_index_st = all_index_st.difference(unslcted_index_st)
    slcted_indexes = list(slcted_index_st)
    rslt = some(ol,*slcted_indexes)
    return(rslt)


def car(ol):
    return(ol[0])

def cdr(ol):
    return(ol[1:])

def head(ol):
    return(ol[0])

def tail(ol):
    return(ol[1:])

def init(ol):
    return(ol[:-1])

def last(ol):
    return(ol[-1])


def preceding(ol,index):
    return(ol[0:index])

def following(ol,index):
    return(ol[index+1:len(ol)])

def via_range(ol,si,ei):
    return(ol[si:ei])


def odds(ol):
    nl = []
    for i in range(0,ol.__len__()):
        if((i%2)!=0):
            nl.append(ol[i])
        else:
            pass
    return(nl)    

def evens(ol):
    nl = []
    for i in range(0,ol.__len__()):
        if((i%2)==0):
            nl.append(ol[i])
        else:
            pass
    return(nl)


def interval(ol,interval,**kwargs):
    si = kwargs['si'] if('si' in kwargs) else 0
    ei = kwargs['ei'] if('ei' in kwargs) else len(ol)
    nl = []
    lngth = ol.__len__()
    for i in range(si,lngth):
        if((i%interval)==0):
            nl.append(ol[i])
        else:
            pass
    return(nl)





import copy
from types import MethodType
from .getset import get_via_pl,get_via_pl2,pl_to_bracket_str
from .sort import bat_sort
from .map import mapv as array_map
from .map import for_eachv
from .crud import append

###
def array_map2(*referls,**kwargs):
    '''
        obseleted just for compatible
        from elist.elist import *
        ol = [1,2,3,4]
        refl1 = ['+','+','+','+']
        refl2 = [7,7,7,7]
        refl3 = ['=','=','=','=']
        def map_func(ele,ref_ele1,ref_ele2,ref_ele3,prefix,suffix):
            s = prefix+': ' + str(ele) + str(ref_ele1) + str(ref_ele2) + str(ref_ele3) + suffix
            return(s)
        ####
        rslt = array_map2(ol,refl1,refl2,refl3,map_func=map_func,map_func_args=['Q','?'])
        pobj(rslt)
    '''
    map_func = kwargs['map_func']
    if('map_func_args' in kwargs):
        map_func_args = kwargs['map_func_args']
    else:
        map_func_args = []
    length = referls.__len__()
    rslt = []
    anum = list(referls)[0].__len__()
    for j in range(0,anum):
        args = []
        for i in range(0,length):
            refl = referls[i]
            args.append(refl[j])
        args.extend(map_func_args)
        v = map_func(*args)
        rslt.append(v)
    return(rslt)



#the below is for nested analysis

def is_leaf(obj):
    '''
        the below is for nested-list
        any type is not list will be treated as a leaf
        empty list will be treated as a leaf
        from elist.elist import *
        is_leaf(1)
        is_leaf([1,2,3])
        is_leaf([])
    '''
    if(isinstance(obj,list)):
        length = obj.__len__()
        if(length == 0):
            return(True)
        else:
            return(False)
    else:
        return(True)

class LevelCache():
    '''
        current level unhandled_data stored in .data
        current level unhandled_desc stored in .desc
        next level unhandled_data stored in .ndata
        next level unhandled_desc stored in .ndesc
    '''
    def help(self):
        print(self.__doc__)
    def __init__(self,**kwargs):
        if('datas' in kwargs):
            datas = kwargs['datas']
        else:
            datas = []
        if('descs' in kwargs):
            descs = kwargs['descs']
        else:
            descs = []
        self.data = [datas]
        self.desc = [descs]
        self.ndata = []
        self.ndesc = []
    def update(self):
        self.data = self.ndata
        self.desc = self.ndesc
        self.ndata = []
        self.ndesc = []
    def __repr__(self):
        print("data: {0}".format(self.data))
        print("desc: {0}".format(self.desc))
        print("ndata: {0}".format(self.ndata))
        print("ndesc: {0}".format(self.ndesc),end='')
        return("")
    def clear(self):
        self.data = []
        self.desc = []
        self.ndata = []
        self.ndesc = []

class StateCache():
    '''
        parent level handled_desc stored in .pdesc_level
        current level handled_desc stored in .desc_level
        过早优化乃万恶之源，之所以有LevelCache 和 StateCache 两个Cache 是为了利用我之前的旧代码，弃之可惜，尾大不掉
    '''
    def help(self):
        print(self.__doc__)
    def __init__(self,root_matrix):
        #there is only one level in root_matrix: level 0
        #there is only one element in root_matrix level 0 :element 0 
        self.matrix = root_matrix
        self.depth = 0
        self.pdesc_level = []
        self.desc_level = self.matrix[0]
    def update(self):
        self.pdesc_level = self.desc_level
        self.matrix.append([])
        self.depth = self.depth + 1
        self.desc_level = self.matrix[self.depth]
    def __repr__(self):
        print("depth: {0}".format(self.depth))
        print("pdesc_level: {0}".format(self.pdesc_level))
        print("desc_level: {0}".format(self.desc_level),end='')
        return("")

def pcache_bind_dynamic_method(pcache,**kwargs):
    '''
    '''
    mn = kwargs['method_name']
    func = kwargs['func']
    method_names = ['get_children_handler','parent_handler','child_begin_handler','leaf_handler','non_leaf_handler','child_end_handler']
    cond = (mn in method_names)
    if(cond):
        pcache.__setattr__(mn, MethodType(func,pcache))
    else:
        pass
    return(pcache)

def init_pcache_handler_inline(kwargs):
    pcache = PointerCache()
    for mn in kwargs:
        cond = (mn in ['get_children_handler','parent_handler','child_begin_handler','leaf_handler','non_leaf_handler','child_end_handler'])
        if(cond):
            func = kwargs[mn]
            pcache.__setattr__(mn, MethodType(func,pcache))
        else:
            pass
    return(pcache)

def pcache_reset_methods(pcache,**kwargs):
    '''
    '''
    def get_children_handler(self,*args):
        '''
            list's children list is self
        '''
        return(self.pdata)
    def parent_handler(self,lcache,i,*args):
        '''
            _update_pdesc_sons_info
        '''
        pdesc = lcache.desc[i]
        pdesc['sons_count'] = self.sibs_len
        pdesc['leaf_son_paths'] = []
        pdesc['non_leaf_son_paths'] = []
        return(pdesc)
    def child_begin_handler(self,scache,*args):
        '''
            _creat_child_desc
            update depth,parent_breadth_path,parent_path,sib_seq,path,lsib_path,rsib_path,lcin_path,rcin_path
        '''
        pdesc = self.pdesc
        depth = scache.depth
        sib_seq = self.sib_seq
        sibs_len = self.sibs_len
        pdesc_level = scache.pdesc_level
        desc = copy.deepcopy(pdesc)
        desc = reset_parent_desc_template(desc)
        desc['depth'] = depth
        desc['parent_breadth_path'] = copy.deepcopy(desc['breadth_path'])
        desc['sib_seq'] = sib_seq
        desc['parent_path'] = copy.deepcopy(desc['path'])
        desc['path'].append(sib_seq)
        update_desc_lsib_path(desc)
        update_desc_rsib_path(desc,sibs_len)
        if(depth == 1):
            pass
        else:
            update_desc_lcin_path(desc,pdesc_level)
            update_desc_rcin_path(desc,sibs_len,pdesc_level)
        return(desc)
    def leaf_handler(self,*args):
        '''#leaf child handler'''
        desc = self.desc
        pdesc = self.pdesc
        desc['leaf'] = True
        desc['sons_count'] = 0
        pdesc['leaf_son_paths'].append(copy.deepcopy(desc['path']))
    def non_leaf_handler(self,lcache):
        '''#nonleaf child handler'''
        desc = self.leaf
        pdesc = self.pdesc
        desc['leaf'] = False
        pdesc['non_leaf_son_paths'].append(copy.deepcopy(desc['path']))
        lcache.ndata.append(self.data)
        lcache.ndesc.append(desc)
    def child_end_handler(self,scache):
        '''
            _upgrade_breadth_info
            update breadth, breadth_path, and add desc to desc_level
        '''
        desc = self.desc
        desc_level = scache.desc_level
        breadth = desc_level.__len__()
        desc['breadth'] = breadth
        desc['breadth_path'].append(breadth)
        desc_level.append(desc)
    funcs = [get_children_handler,parent_handler,child_begin_handler,leaf_handler,non_leaf_handler,child_end_handler]
    method_names = ['get_children_handler','parent_handler','child_begin_handler','leaf_handler','non_leaf_handler','child_end_handler']
    for i in range(0,funcs.__len__()):
        mn = method_names[i]
        func = funcs[i]
        pcache.__setattr__(mn, MethodType(func,pcache))
    return(pcache)

class PointerCache():
    '''
    '''
    def get_children_handler(self,*args):
        '''
            list's children list is self
        '''
        return(self.pdata)
    def parent_handler(self,lcache,i,*args):
        '''
            _update_pdesc_sons_info
        '''
        pdesc = lcache.desc[i]
        pdesc['sons_count'] = self.sibs_len
        pdesc['leaf_son_paths'] = []
        pdesc['non_leaf_son_paths'] = []
        pdesc['leaf_descendant_paths'] = []
        pdesc['non_leaf_descendant_paths'] = []
        return(pdesc)
    def child_begin_handler(self,scache,*args):
        '''
            _creat_child_desc
            update depth,parent_breadth_path,parent_path,sib_seq,path,lsib_path,rsib_path,lcin_path,rcin_path
        '''
        pdesc = self.pdesc
        depth = scache.depth
        sib_seq = self.sib_seq
        sibs_len = self.sibs_len
        pdesc_level = scache.pdesc_level
        desc = copy.deepcopy(pdesc)
        desc = reset_parent_desc_template(desc)
        desc['depth'] = depth
        desc['parent_breadth_path'] = copy.deepcopy(desc['breadth_path'])
        desc['sib_seq'] = sib_seq
        desc['parent_path'] = copy.deepcopy(desc['path'])
        desc['path'].append(sib_seq)
        update_desc_lsib_path(desc)
        update_desc_rsib_path(desc,sibs_len)
        if(depth == 1):
            pass
        else:
            update_desc_lcin_path(desc,pdesc_level)
            update_desc_rcin_path(desc,sibs_len,pdesc_level)
        return(desc)
    def leaf_handler(self,*args):
        '''#leaf child handler'''
        desc = self.desc
        pdesc = self.pdesc
        desc['leaf'] = True
        desc['sons_count'] = 0
        pdesc['leaf_son_paths'].append(copy.deepcopy(desc['path']))
        pdesc['leaf_descendant_paths'].append(copy.deepcopy(desc['path']))
    def non_leaf_handler(self,lcache):
        '''#nonleaf child handler'''
        desc = self.desc
        pdesc = self.pdesc
        desc['leaf'] = False
        pdesc['non_leaf_son_paths'].append(copy.deepcopy(desc['path']))
        pdesc['non_leaf_descendant_paths'].append(copy.deepcopy(desc['path']))
        lcache.ndata.append(self.data)
        lcache.ndesc.append(desc)
    def child_end_handler(self,scache):
        '''
            _upgrade_breadth_info
            update breadth, breadth_path, and add desc to desc_level
        '''
        desc = self.desc
        desc_level = scache.desc_level
        breadth = desc_level.__len__()
        desc['breadth'] = breadth
        desc['breadth_path'].append(breadth)
        desc_level.append(desc)
    def update_pdesc(self,lcache,i):
        self.unhandled_seq = i
        self.pdata = lcache.data[i]
        self.children = self.get_children_handler(*self.get_children_handler_args)
        self.sibs_len = self.children.__len__()
        self.pdesc = self.parent_handler(lcache,i,*self.parent_handler_args)
    def update_desc(self,lcache,scache,sib_seq):
        self.sib_seq = sib_seq
        self.data = self.children[self.sib_seq]
        self.desc = self.child_begin_handler(scache)
        if(is_leaf(self.data)):
            self.leaf_handler()
        else:
            self.non_leaf_handler(lcache)
        self.child_end_handler(scache)
    def help(self):
        print(self.__doc__)
    def __init__(self,**kwargs):
        if('get_children_handler_args' in kwargs):
            self.get_children_handler_args = kwargs['get_children_handler_args']
        else:
            self.get_children_handler_args = []
        if('parent_handler_args' in kwargs):
            self.parent_handler_args = kwargs['parent_handler_args']
        else:
            self.parent_handler_args = []
        if('child_begin_handler_args' in kwargs):
            self.child_begin_handler_args = kwargs['child_begin_handler_args']
        else:
            self.child_begin_handler_args = []
        if('leaf_handler_args' in kwargs):
            self.leaf_handler_args = kwargs['leaf_handler_args']
        else:
            self.leaf_handler_args = []
        if('non_leaf_handler_args' in kwargs):
            self.non_leaf_handler_args = kwargs['non_leaf_handler_args']
        else:
            self.non_leaf_handler_args = []
        if('child_end_handler_args' in kwargs):
            self.child_end_handler_args = kwargs['child_end_handler_args']
        else:
            self.child_end_handler_args = []
        self.unhandled_seq = None
        self.children = None
        self.sibs_len = None
        self.pdesc = None
        self.sib_seq = None
        self.data = None
        self.desc = None

##the below is for each element desc handle
def new_ele_description(**kwargs):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        root_desc = new_ele_description(leaf=False,depth=0,breadth_path=[],path=[],parent_path=[],parent_breadth_path=[])
        pobj(root_desc)
        #None means not handled
    '''
    desc = {
        'leaf':None,
        'depth':None,
        'breadth':None,
        'breadth_path':None,
        'sib_seq':None,
        'path':None,
        'parent_path':None,
        'parent_breadth_path':None,
        'lsib_path':None,
        'rsib_path':None,
        'lcin_path':None,
        'rcin_path':None,
        'sons_count':None,
        'leaf_son_paths':None,
        'non_leaf_son_paths':None,
        'leaf_descendant_paths':None,
        'non_leaf_descendant_paths':None,
        'flat_offset':None,
        'flat_len':None
    }
    for key in kwargs:
        desc[key.lower()] = kwargs[key]
    return(desc)

def root_list(*args):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        root_list([1],2,[1,2,3])
    '''
    return(list(args))

def init_desc_matrix(l):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        l = [1,[4],2,[3,[5,6]]]
        desc_matrix = init_desc_matrix(l)
        pobj(desc_matrix)
    '''
    leaf = is_leaf(l)
    root_desc = new_ele_description(leaf=leaf,depth=0,breadth_path=[],path=[],parent_path=[],parent_breadth_path=[])
    if(leaf):
        root_desc['sons_count'] = 0
    else:
        pass
    desc_matrix = [
        [root_desc]
    ]
    return(desc_matrix)

def reset_parent_desc_template(desc):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        pobj(desc)
        tem = reset_parent_desc_template(desc)
        pobj(tem)
        #only inherit path  and breadth_path
    '''
    tem = new_ele_description()
    tem['path'] = desc['path']
    tem['breadth_path'] = desc['breadth_path']
    return(tem)

def _init_unhandled(l,inited_matrix):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        l = [1,[4],2,[3,[5,6]]]
        desc_matrix = init_desc_matrix(l)
        unhandled = _init_unhandled(l,desc_matrix)
        unhandled_data = unhandled['data']
        unhandled_desc = unhandled['desc']
        unhandled_data[0]
        unhandled_desc[0]
        unhandled_data[1]
        unhandled_desc[1]
    '''
    root_desc = inited_matrix[0][0]
    unhandled = {'data':[],'desc':[]}
    length = l.__len__()
    root_desc['sons_count'] = length
    root_desc['leaf_son_paths'] = []
    root_desc['non_leaf_son_paths'] = []    
    if(length == 0):
        pass
    else:
        inited_matrix.append([])
        level = inited_matrix[1]
        for i in range(0,length):
            child = l[i]
            desc = copy.deepcopy(root_desc)
            desc = reset_parent_desc_template(desc)
            desc['depth'] = 1
            desc['breadth'] = i
            desc['parent_breadth_path'] = copy.deepcopy(desc['breadth_path'])
            desc['breadth_path'].append(i)
            desc['sib_seq'] = i
            desc['parent_path'] = copy.deepcopy(desc['path'])
            desc['path'].append(i)
            if(i==0):
                pass
            else:
                desc['lsib_path'] = [i-1]
            if(i == (length - 1)):
                pass
            else:
                desc['rsib_path'] = [i+1]
            if(is_leaf(child)):
                desc['leaf'] = True
                desc['sons_count'] = 0
                root_desc['leaf_son_paths'].append(copy.deepcopy(desc['path']))
            else:
                desc['leaf'] = False
                root_desc['non_leaf_son_paths'].append(copy.deepcopy(desc['path']))
                unhandled['data'].append(child)
                unhandled['desc'].append(desc)
            level.append(desc)
    return(unhandled)

def update_desc_lsib_path(desc):
    '''
        leftSibling
        previousSibling
        leftSib
        prevSib
        lsib
        psib
        
        have the same parent,and on the left
    '''
    if(desc['sib_seq']>0):
        lsib_path = copy.deepcopy(desc['path'])
        lsib_path[-1] = desc['sib_seq']-1
        desc['lsib_path'] = lsib_path
    else:
        pass
    return(desc)

def update_desc_rsib_path(desc,sibs_len):
    '''
        rightSibling
        nextSibling
        rightSib
        nextSib
        rsib
        nsib
        
        have the same parent,and on the right
    '''
    if(desc['sib_seq']<(sibs_len-1)):
        rsib_path = copy.deepcopy(desc['path'])
        rsib_path[-1] = desc['sib_seq']+1
        desc['rsib_path'] = rsib_path
    else:
        pass
    return(desc)

def update_desc_lcin_path(desc,pdesc_level):
    '''
        leftCousin
        previousCousin
        leftCin
        prevCin
        lcin
        pcin
        
        parents are neighbors,and on the left
    '''
    parent_breadth = desc['parent_breadth_path'][-1]
    if(desc['sib_seq']==0):
        if(parent_breadth==0):
            pass
        else:
            parent_lsib_breadth = parent_breadth - 1
            plsib_desc = pdesc_level[parent_lsib_breadth]
            if(plsib_desc['leaf']):
                pass
            else:
                lcin_path = copy.deepcopy(plsib_desc['path'])
                lcin_path.append(plsib_desc['sons_count'] - 1)
                desc['lcin_path'] = lcin_path
    else:
        pass
    return(desc)

def update_desc_rcin_path(desc,sibs_len,pdesc_level):
    '''
        rightCousin
        nextCousin
        rightCin
        nextCin
        rcin
        ncin
        
        parents are neighbors,and on the right
    '''
    psibs_len = pdesc_level.__len__()
    parent_breadth = desc['parent_breadth_path'][-1]
    if(desc['sib_seq']==(sibs_len - 1)):
        if(parent_breadth==(psibs_len -1)):
            pass
        else:
            parent_rsib_breadth = parent_breadth + 1
            prsib_desc = pdesc_level[parent_rsib_breadth]
            #because from left to right to handle each level
            #sons_count will only be updated in the next-round 
            if(prsib_desc['leaf']):
                pass
            else:
                rcin_path = copy.deepcopy(prsib_desc['path'])
                rcin_path.append(0)
                desc['rcin_path'] = rcin_path
    else:
        pass
    return(desc)

def scan(l,**kwargs):
    '''
        from elist.elist import *
        from elist.jprint import pobj
        l = [1,[4],2,[3,[5,6]]]
        desc = description(l)
        l = [1,2,[4],[3,[5,6]]]
        desc = description(l)
    '''
    if('itermode' in kwargs):
        itermode = True
    else:
        itermode = False
    ####level ==  0
    desc_matrix = init_desc_matrix(l)
    if(desc_matrix[0][0]['leaf'] == True):
        return(desc_matrix)
    else:
        pass
    ####cache
    lcache=LevelCache(datas=l,descs=desc_matrix[0][0])
    scache=StateCache(desc_matrix)
    pcache = init_pcache_handler_inline(kwargs)
    ####level > 0
    while(lcache.data.__len__() > 0):
        #add next desc_level 
        scache.update()
        for unhandled_seq in range(0,lcache.data.__len__()):
            #handle parent
            pcache.update_pdesc(lcache,unhandled_seq)
            for sib_seq in range(0,pcache.sibs_len):
                #handle child
                pcache.update_desc(lcache,scache,sib_seq)
        #update level lcache
        lcache.update()
    return(desc_matrix)

class DescMatrix():
    '''
    '''
    def __init__(self,matrix):
        self.matrix = matrix
    @classmethod
    def loc(cls,desc):
        return([desc['depth'],desc['breadth']])
    @classmethod
    def ploc(cls,desc):
        if(desc['parent_breadth_path'] == []):
            col = 0
        else:
            col = desc['parent_breadth_path'][-1]
        if(desc['depth'] == 0):
            row = 0
        else:
            row = desc['depth']-1
        return([row,col])
    def pdesc(self,desc):
        pd = get_via_pl(self.matrix,self.ploc(desc))
        return(pd)

def fullfill_descendants_info(desc_matrix):
    '''
       flat_offset
    '''
    pathloc_mapping = {}
    locpath_mapping = {}
    #def leaf_handler(desc,pdesc,offset):
    def leaf_handler(desc,pdesc):
        #desc['flat_offset'] = (offset,offset+1)
        desc['non_leaf_son_paths'] = []
        desc['leaf_son_paths'] = []
        desc['non_leaf_descendant_paths'] = []
        desc['leaf_descendant_paths'] = []
        desc['flat_len'] = 1
        if(pdesc['flat_len']):
            pdesc['flat_len'] = pdesc['flat_len'] + 1
        else:
            pdesc['flat_len'] = 1
    #def non_leaf_handler(desc,pdesc,offset):
    def non_leaf_handler(desc,pdesc):
        #desc['flat_offset'] = (offset,offset+desc['flat_len'])
        pdesc['non_leaf_descendant_paths'].extend(copy.deepcopy(desc['non_leaf_descendant_paths']))
        pdesc['leaf_descendant_paths'].extend(copy.deepcopy(desc['leaf_descendant_paths']))
        if(pdesc['flat_len']):
            pdesc['flat_len'] = pdesc['flat_len'] + desc['flat_len']
        else:
            pdesc['flat_len'] = desc['flat_len']
    def fill_path_mapping(desc):
        pmk = tuple(desc['path'])
        pmv = tuple(DescMatrix.loc(desc))
        pathloc_mapping[pmk] = pmv
        locpath_mapping[pmv] = pmk
    dm = DescMatrix(desc_matrix)
    depth = desc_matrix.__len__()
    desc_level = desc_matrix[depth - 1]
    length = desc_level.__len__()
    #the last level
    #offset = 0
    for j in range(length - 1,-1,-1):
        desc = desc_level[j]
        fill_path_mapping(desc)
        pdesc = dm.pdesc(desc)
        leaf_handler(desc,pdesc)
        #leaf_handler(desc,pdesc,offset)
        #offset = offset + 1
    for i in range(depth-2,0,-1):
        #offset = 0
        desc_level = desc_matrix[i]
        length = desc_level.__len__()
        for j in range(length-1,-1,-1):
            desc = desc_level[j]
            fill_path_mapping(desc)
            pdesc = dm.pdesc(desc)
            if(desc['leaf']):
                leaf_handler(desc,pdesc)
                #leaf_handler(desc,pdesc,offset)
                #offset = offset + 1
            else:
                non_leaf_handler(desc,pdesc)
                #non_leaf_handler(desc,pdesc,offset)
                #offset = offset + desc['flat_len']
    desc_matrix[0][0]['flat_offset'] = (0,desc_matrix[0][0]['flat_len'])
    for i in range(0,depth-1):
        pdesc_level = desc_matrix[i]
        length = pdesc_level.__len__()
        for j in range(0,length):
            pdesc = pdesc_level[j]
            si = pdesc['flat_offset'][0]
            for i in range(0,pdesc['sons_count']):
                spl = append(pdesc['path'],i,mode='new')
                pk = tuple(spl)
                locx,locy = pathloc_mapping[pk]
                son = desc_matrix[locx][locy]
                ei = si + son['flat_len']
                son['flat_offset'] = (si,ei)
                si = ei
    return(desc_matrix,pathloc_mapping,locpath_mapping)

####from elist.jprint

def get_block_op_pairs(pairs_str):
    '''
        # >>> get_block_op_pairs("{}[]")  
        # {1: ('{', '}'), 2: ('[', ']')}
        # >>> get_block_op_pairs("{}[]()")
        # {1: ('{', '}'), 2: ('[', ']'), 3: ('(', ')')}
        # >>> get_block_op_pairs("{}[]()<>")
        # {1: ('{', '}'), 2: ('[', ']'), 3: ('(', ')'), 4: ('<', '>')}
    '''
    pairs_str_len = pairs_str.__len__()
    pairs_len = pairs_str_len // 2
    pairs_dict = {}
    for i in range(1,pairs_len +1):
        pairs_dict[i] = pairs_str[i*2-2],pairs_str[i*2-1]
    return(pairs_dict)

def is_lop(ch,block_op_pairs_dict=get_block_op_pairs('{}[]()')):
    '''
    # is_lop('{',block_op_pairs_dict)
    # is_lop('[',block_op_pairs_dict)
    # is_lop('}',block_op_pairs_dict)
    # is_lop(']',block_op_pairs_dict)
    # is_lop('a',block_op_pairs_dict)
    '''
    for i in range(1,block_op_pairs_dict.__len__()+1):
        if(ch == block_op_pairs_dict[i][0]):
            return(True)
        else:
            pass
    return(False)

def is_rop(ch,block_op_pairs_dict=get_block_op_pairs('{}[]()')):
    '''
        # is_rop('{',block_op_pairs_dict)
        # is_rop('[',block_op_pairs_dict)
        # is_rop('}',block_op_pairs_dict)
        # is_rop(']',block_op_pairs_dict)
        # is_rop('a',block_op_pairs_dict)
    '''
    for i in range(1,block_op_pairs_dict.__len__()+1):
        if(ch == block_op_pairs_dict[i][1]):
            return(True)
        else:
            pass
    return(False)

def get_next_char_level_in_j_str(curr_lv,curr_seq,j_str,block_op_pairs_dict=get_block_op_pairs("{}[]()")):
    ''' the first-char is level-1
        when current is  non-op, next-char-level = curr-level
        when current is  lop,  non-paired-rop-next-char-level = lop-level+1;
        when current is  lop,  paired-rop-next-char-level = lop-level
        when current is  rop,  next-char-level = rop-level - 1
        # {"key_4_UF0aJJ6v": "value_1", "key_2_Hd0t": ["value_16", "value_8", "value_8", "value_15", "value_14", "value_19", {......
        # 122222222222222222222222222222222222222222222333333333333333333333333333333333333333333333333333333333333333333333334......
        # {\n"key_4_UF0aJJ6v": "value_1", \n"key_2_Hd0t": [\n"value_16", \n"value_8", \n"value_8", \n"value_15", \n"value_14", \n"value_19",...... 
        # 1 222222222222222222222222222222 2222222222222222 3333333333333 333333333333 333333333333 3333333333333 3333333333333 3333333333333...... 
        '''
    curr_ch = j_str[curr_seq]
    next_ch = j_str[curr_seq + 1]
    cond = 0
    for i in range(1,block_op_pairs_dict.__len__()+1):
        if(curr_ch == block_op_pairs_dict[i][0]):
            if(next_ch == block_op_pairs_dict[i][1]):
                next_lv = curr_lv               
            else:
                next_lv = curr_lv + 1
            cond = 1
            break
        elif(curr_ch == block_op_pairs_dict[i][1]):
            if(is_rop(next_ch,block_op_pairs_dict)):
                next_lv = curr_lv - 1
            else:
                next_lv = curr_lv
            cond = 1
            break
        else:
            pass
    if(cond == 1):
        pass
    elif(is_rop(next_ch,block_op_pairs_dict)):
        next_lv = curr_lv - 1
    else:    
        next_lv = curr_lv
    curr_lv = next_lv
    curr_seq = curr_seq + 1
    return(curr_lv,curr_lv,curr_seq)

def get_j_str_lvs_dict(j_str,block_op_pairs_dict=get_block_op_pairs("{}[]()")):
    j_str_len = j_str.__len__()
    j_str_lvs_dict = {}
    if( j_str_len == 0):
        j_str_lvs_dict = {}
    elif(j_str_len == 1):
        j_str_lvs_dict = {0:1}
    else:
        curr_lv = 1
        j_str_lvs_dict = {0:1}
        seq = 1
        curr_seq = 0
        while(curr_seq < j_str_len - 1):
            level,curr_lv,curr_seq = get_next_char_level_in_j_str(curr_lv,curr_seq,j_str,block_op_pairs_dict)
            j_str_lvs_dict[seq] =level
            seq = seq + 1
    return(j_str_lvs_dict)



####from elist.utils
def str_display_width(s):
    '''
        from elist.utils import *
        str_display_width('a')
        str_display_width('去')
    '''
    s= str(s)
    width = 0
    len = s.__len__()
    for i in range(0,len):
        sublen = s[i].encode().__len__()
        sublen = int(sublen/2 + 1/2)
        width = width + sublen
    return(width)

####from elist.ltdict
def ltdict2list(ltdict):
    l = []
    length = ltdict.__len__()
    for i in range(0,length):
        l.append(ltdict[i])
    return(l)

####beautiful display
def spacize(s,lvnum):
    lvs = get_j_str_lvs_dict(s)
    lvs = ltdict2list(lvs)
    sl=list(s)
    length = sl.__len__()
    rslt =''
    for i in range(0,length):
        if(lvs[i]>=lvnum):
            rslt = rslt + sl[i]
        else:
            rslt = rslt + chr(32)*str_display_width(sl[i])
    return(rslt)

def table(l,depth,**kwargs):
    if('no_return' in kwargs):
        no_return = kwargs['no_return']
    else:
        no_return = True
    s = l.__str__()
    rslt = ''
    for i in range(1,depth+1):
        rslt = rslt + spacize(s,i) + '\n'
    rslt = rslt[:-1]
    if(no_return):
        print(rslt)
    else:
        return(rslt)

####mat func for description matrix

def matrix_map(mat,map_func,map_func_args=[]):
    '''
        mat = [
            [1,2,3],
            [4,5,6],
            [7,8,9]
        ]
        
        def map_func(value,indexr,indexc,prefix,suffix):
            msg = prefix + str((indexr,indexc)) + " : "+str(value) + suffix
            return(msg)
        
        mmat = matrix_map(mat,map_func,map_func_args=["<",">"])
        
        mmat 
    '''
    mmat = []
    for i in range(0,mat.__len__()):
        level = mat[i]
        mmat.append([])
        for j in range(0,level.__len__()):
            value = level[j]
            indexr = i
            indexc = j
            ele = map_func(value,indexr,indexc,*map_func_args)
            mmat[i].append(ele)
    return(mmat)

def is_matrix(m,**kwargs):
    def cond_func(ele,lngth):
        cond = (ele.__len__() == lngth)
        return(cond)
    cond_1 = every(m,isinstance,list)
    if(cond_1):
        if('mode' in kwargs):
            mode = kwargs['mode']
        else:
            mode = 'strict' 
        if(mode == 'strict'):
            try:
                lngth = m[0].__len__()
                cond_2 = every(m,cond_func,lngth)
            except:
                return(False)
            else:
                pass
        else:
            cond_2 = True
        return(cond_1 & cond_2)
    else:
        return(True)


#

#dfs depth-first-search trace
def get_dfs(l):
    '''
        l = ['v_7', 'v_3', 'v_1', 'v_4', ['v_4', 'v_2'], 'v_5', 'v_6', 'v_1', 'v_6', 'v_7', 'v_5', ['v_4', ['v_1', 'v_8', 'v_3', 'v_4', 'v_2', 'v_7', [['v_3', 'v_2'], 'v_4', 'v_5', 'v_1', 'v_3', 'v_1', 'v_2', 'v_5', 'v_8', 'v_8', 'v_7'], 'v_5', 'v_8', 'v_7', 'v_1', 'v_5'], 'v_6'], 'v_4', 'v_5', 'v_8', 'v_5']
        dfs = get_dfs(l)
    '''
    ltree = ListTree(l)
    dfs = ltree.tree()
    return(dfs)


#wfs  width-first-search trace
#wfsmat width-first-search matrix

def get_wfsmat(l):
    '''
       l = ['v_7', 'v_3', 'v_1', 'v_4', ['v_4', 'v_2'], 'v_5', 'v_6', 'v_1', 'v_6', 'v_7', 'v_5', ['v_4', ['v_1', 'v_8', 'v_3', 'v_4', 'v_2', 'v_7', [['v_3', 'v_2'], 'v_4', 'v_5', 'v_1', 'v_3', 'v_1', 'v_2', 'v_5', 'v_8', 'v_8', 'v_7'], 'v_5', 'v_8', 'v_7', 'v_1', 'v_5'], 'v_6'], 'v_4', 'v_5', 'v_8', 'v_5']
       get_wfs(l)
    '''
    ltree = ListTree(l)
    vdescmat = ltree.desc
    wfsmat = matrix_map(vdescmat,lambda v,ix,iy:v['path'])
    wfsmat.pop(0)
    return(wfsmat)



def mat2wfs(wfsmat):
    trace = []
    for i in range(0,wfsmat.__len__()):
        trace.extend(wfsmat[i])
    return(trace)

def wfs2mat(wfs):
    '''
        wfs = [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [10], [11], [12], [13], [14], [15], [4, 0], [4, 1], [11, 0], [11, 1], [11, 2], [11, 1, 0], [11, 1, 1], [11, 1, 2], [11, 1, 3], [11, 1, 4], [11, 1, 5], [11, 1, 6], [11, 1, 7], [11, 1, 8], [11, 1, 9], [11, 1, 10], [11, 1, 11], [11, 1, 6, 0], [11, 1, 6, 1], [11, 1, 6, 2], [11, 1, 6, 3], [11, 1, 6, 4], [11, 1, 6, 5], [11, 1, 6, 6], [11, 1, 6, 7], [11, 1, 6, 8], [11, 1, 6, 9], [11, 1, 6, 10], [11, 1, 6, 0, 0], [11, 1, 6, 0, 1]]
    '''
    wfsmat = []
    depth = 0
    level = filter(wfs,lambda ele:ele.__len__()==1)
    while(level.__len__()>0):
        wfsmat.append([])
        wfsmat[depth] = level
        depth = depth+1
        level = filter(wfs,lambda ele:ele.__len__()==depth+1)
    return(wfsmat)


def get_wfs(l):
    wfsmat = get_wfsmat(l)
    wfs =  mat2wfs(wfsmat)
    return(wfs)


def dfs2wfsmat(dfs):
    '''
        dfs = [[0], [1], [2], [3], [4], [4, 0], [4, 1], [5], [6], [7], [8], [9], [10], [11], [11, 0], [11, 1], [11, 1, 0], [11, 1, 1], [11, 1, 2], [11, 1, 3], [11, 1, 4], [11, 1, 5], [11, 1, 6], [11, 1, 6, 0], [11, 1, 6, 0, 0], [11, 1, 6, 0, 1], [11, 1, 6, 1], [11, 1, 6, 2], [11, 1, 6, 3], [11, 1, 6, 4], [11, 1, 6, 5], [11, 1, 6, 6], [11, 1, 6, 7], [11, 1, 6, 8], [11, 1, 6, 9], [11, 1, 6, 10], [11, 1, 7], [11, 1, 8], [11, 1, 9], [11, 1, 10], [11, 1, 11], [11, 2], [12], [13], [14], [15]]
        
        dfs2wfs(dfs)
    '''
    wfsmat = []
    depth = 0
    level = filter(dfs,lambda ele:ele.__len__()==1)
    while(level.__len__()>0):
        wfsmat.append([])
        wfsmat[depth] = level
        depth = depth+1
        level = filter(dfs,lambda ele:ele.__len__()==depth+1)
    return(wfsmat)


     


def wfsmat2dfs(wfsmat):
    '''
        wfs = [[[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [10], [11], [12], [13], [14], [15]], [[4, 0], [4, 1], [11, 0], [11, 1], [11, 2]], [[11, 1, 0], [11, 1, 1], [11, 1, 2], [11, 1, 3], [11, 1, 4], [11, 1, 5], [11, 1, 6], [11, 1, 7], [11, 1, 8], [11, 1, 9], [11, 1, 10], [11, 1, 11]], [[11, 1, 6, 0], [11, 1, 6, 1], [11, 1, 6, 2], [11, 1, 6, 3], [11, 1, 6, 4], [11, 1, 6, 5], [11, 1, 6, 6], [11, 1, 6, 7], [11, 1, 6, 8], [11, 1, 6, 9], [11, 1, 6, 10]], [[11, 1, 6, 0, 0], [11, 1, 6, 0, 1]]]
        wfs2dfs(wfs)
        
    '''
    dfs = mat2wfs(wfsmat)
    dfs.sort()
    return(dfs)




def dfs2wfs(dfs):
    wfsmat = dfs2wfsmat(dfs)
    wfs = mat2wfs(wfsmat)
    return(wfs)

def wfs2dfs(wfs):
    wfsmat = wfs2mat(wfs)
    dfs = wfsmat2dfs(wfsmat)
    return(dfs)

####


class ListTree():
    '''
        
        
        ltree.parent_path(3,1,0)
        ltree.parent(3,1,0)
        
        ltree.son_paths(3)
        ltree.sons(3)
        ltree.son_paths(3,leaf_only=True)
        ltree.son_paths(3,non_leaf_only=True)
        ltree.sons(3,leaf_only=True)
        ltree.sons(3,non_leaf_only=True)
        
        ltree.descendant_paths(3)
        ltree.descendants(3)
        ltree.descendant_paths(3,from_lv=3)
        ltree.descendants(3,from_lv=3)
        ltree.descendant_paths(3,from_lv=2,to_lv=2)
        ltree.descendants(3,from_lv=2,to_lv=2)
        ltree.descendant_paths(3,leaf_only=True)
        ltree.descendants(3,leaf_only=True)
        ltree.descendant_paths(3,non_leaf_only=True)
        ltree.descendants(3,non_leaf_only=True)
        
        ltree.ancestor_paths(3,1,0)
        ltree.ancestors(3,1,0)
    '''
    def __init__(self,l):
        self.list = l
        self.desc = scan(l)
        self.desc,self.pathloc_mapping,self.locpath_mapping= fullfill_descendants_info(self.desc)
        self.depth = self.desc.__len__()
        self.maxLevelWidth = max(array_map(self.desc,len))
        self.flatWidth = self.desc[0][0]['flat_len']
        self.total = self.desc[0][0]['leaf_descendant_paths'].__len__() + self.desc[0][0]['non_leaf_descendant_paths'].__len__()
        self.trace = self.tree(show=False)
        self.prevSibling = self.lsib
        self.prevSibPath = self.lsib_path
        self.nextSibling = self.rsib
        self.nextSibPath = self.rsib_path
        self.precedingSibPaths = self.preceding_sib_paths
        self.precedingSibs = self.preceding_sibs
        self.followingSibPaths = self.following_sib_paths
        self.followingSibs = self.following_sibs
        self.someSibPaths = self.some_sib_paths
        self.someSibs = self.some_sibs
        self.whichSibPath = self.which_sib_path
        self.whichSib = self.which_sib
        self.showlog = None
    def __repr__(self):
        s = table(self.list,self.depth,no_return=0)
        showl = s.split('\n')
        self.showlog = showl
        return(s)
    def tree(self,**kwargs):
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
            prompt = 'leaf_only'
        else:
            leaf_only = False
            prompt = ''
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
            prompt = 'non_leaf_only'
        else:
            non_leaf_only = False
            prompt = ''
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = self.depth
        if('show' in kwargs):
            show = kwargs['show']
        else:
            show = True
        lpls = copy.deepcopy(self.desc[0][0]['leaf_descendant_paths'])
        nlpls = copy.deepcopy(self.desc[0][0]['non_leaf_descendant_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        nrslt = []
        for i in range(0,rslt.__len__()):
            pl = rslt[i]
            length = pl.__len__()
            cond1 = (length >= from_lv)
            cond2 = (length <= to_lv)
            if(cond1 & cond2):
                nrslt.append(pl)
            else:
                pass
        showl = array_map(nrslt,pl_to_bracket_str)
        nrslt,showl = bat_sort(nrslt,nrslt,showl)
        if(show):
            for_eachv(showl,print)
            self.showlog = ['tree -'+prompt+' :']
            self.showlog.extend(showl)
        else:
            pass
        return(nrslt)
    def level(self,lvnum,**kwargs):
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
            prompt = 'leaf_only'
        else:
            leaf_only = False
            prompt = ''
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
            prompt = 'non_leaf_only'
        else:
            non_leaf_only = False
            prompt = ''
        desc_level = self.desc[lvnum]
        lpls = []
        nlpls = []
        for i in range(0,desc_level.__len__()):
            desc = desc_level[i]
            pathlist = copy.deepcopy(desc['path'])
            if(desc['leaf']):
                lpls.append(pathlist)
            else:
                nlpls.append(pathlist)
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        showl = array_map(rslt,pl_to_bracket_str)
        rslt,showl = bat_sort(rslt,rslt,showl)
        for_eachv(showl,print)
        self.showlog = ['level -' +prompt+' ' +str(lvnum)+' :']
        self.showlog.extend(showl)
    def flatten(self):
        lpls = self.tree(leaf_only=True,show=False)
        flat = array_map(lpls,get_via_pl2,self.list)
        return(flat)
    def include(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        try:
            get_via_pl(self.list,pl)
        except:
            return(False)
        else:
            return(True)
    def __getitem__(self,*sibseqs):
        #this is a trick for __getitem__
        sibseqs = sibseqs[0]
        return(get_via_sibseqs(self.list,*sibseqs))
    def loc(self,*sibseqs):
        pl = list(sibseqs)
        pk = tuple(pl)
        loc = self.pathloc_mapping[pk]
        return(list(loc))
    def path(self,locx,locy):
        loc = (locx,locy)
        pl = self.locpath_mapping[loc]
        pl = list(pl)
        return(pl)
    def path2loc(self,pathlist):
        pl = pathlist
        pk = tuple(pl)
        loc = self.pathloc_mapping[pk]
        return(list(loc))
    def loc2path(self,loc):
        loc = tuple(loc)
        pl = self.locpath_mapping[loc]
        pl = list(pl)
        return(pl)
    @classmethod
    def showroute(cls,arr):
        def arrow(ele):
            return(str(ele)+' ->')
        arr = array_map(arr,arrow)
        for_eachv(arr,print)
        return(arr)
    def dig(self,howmanysteps=None):
        if(howmanysteps):
            pass
        else:
            howmanysteps = self.total
        self.showlog = ['dig -steps '+howmanysteps+' :']
        self.showlog.extend(self.showroute(self.trace[:howmanysteps]))
        return(self.trace[:howmanysteps])
    def parent(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        loc = self.path2loc(pl)
        ppl = self.desc[loc[0]][loc[1]]['parent_path']
        value = get_via_pl(self.list,ppl)
        return(value)
    def parent_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        return(ppl)
    def son_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        lpls = copy.deepcopy(self.desc[locx][locy]['leaf_son_paths'])
        nlpls = copy.deepcopy(self.desc[locx][locy]['non_leaf_son_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        rslt,= bat_sort(rslt,rslt)
        return(rslt)
    def sons(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        lpls = copy.deepcopy(self.desc[locx][locy]['leaf_son_paths'])
        nlpls = copy.deepcopy(self.desc[locx][locy]['non_leaf_son_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        rslt,= bat_sort(rslt,rslt)
        rslt = array_map(rslt,get_via_pl2,self.list)
        return(rslt)
    def descendant_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = self.depth
        locx,locy = tuple(self.path2loc(pl))
        lpls = copy.deepcopy(self.desc[locx][locy]['leaf_descendant_paths'])
        nlpls = copy.deepcopy(self.desc[locx][locy]['non_leaf_descendant_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        nrslt = []
        for i in range(0,rslt.__len__()):
            pl = rslt[i]
            length = pl.__len__()
            cond1 = (length >= from_lv)
            cond2 = (length <= to_lv)
            if(cond1 & cond2):
                nrslt.append(pl)
            else:
                pass
        nrslt, = bat_sort(nrslt,nrslt)
        return(nrslt)
    def descendants(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        lpls = copy.deepcopy(self.desc[locx][locy]['leaf_descendant_paths'])
        nlpls = copy.deepcopy(self.desc[locx][locy]['non_leaf_descendant_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = self.depth
        nrslt = []
        for i in range(0,rslt.__len__()):
            pl = rslt[i]
            length = pl.__len__()
            cond1 = (length >= from_lv)
            cond2 = (length <= to_lv)
            if(cond1 & cond2):
                nrslt.append(pl)
            else:
                pass
        nrslt, = bat_sort(nrslt,nrslt)
        nrslt = array_map(nrslt,get_via_pl2,self.list)
        return(nrslt)
    @classmethod
    def ancestlize(cls,l,**kwargs):
        length = l.__len__()
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = length - 2
        nrslt = []
        si = from_lv - 1
        ei = to_lv + 1
        for i in range(si,ei):
            pl = l[:(i+1)]
            nrslt.append(pl)
        return(nrslt)
    def ancestor_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        p = copy.deepcopy(self.desc[locx][locy]['path'])
        anps = self.ancestlize(p,from_lv=from_lv,to_lv=to_lv)
        return(anps)
    def ancestors(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        p = copy.deepcopy(self.desc[locx][locy]['path'])
        anps = self.ancestlize(p,from_lv=from_lv,to_lv=to_lv)
        ans = array_map(anps,get_via_pl2,self.list)
        return(ans)
    def lsib_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        lsibp = copy.deepcopy(self.desc[locx][locy]['lsib_path'])
        return(lsibp)
    def lsib(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        lsibp = copy.deepcopy(self.desc[locx][locy]['lsib_path'])
        lsibv = get_via_pl(self.list,lsibp) 
        return(lsibv)
    def rsib_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        rsibp = copy.deepcopy(self.desc[locx][locy]['rsib_path'])
        return(rsibp)
    def rsib(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        rsibp = copy.deepcopy(self.desc[locx][locy]['rsib_path'])
        rsibv = get_via_pl(self.list,rsibp) 
        return(rsibv)
    def lcin_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        lcinp = copy.deepcopy(self.desc[locx][locy]['lcin_path'])
        return(lcinp)
    def lcin(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        lcinp = copy.deepcopy(self.desc[locx][locy]['lcin_path'])
        lcinv = get_via_pl(self.list,lcinp) 
        return(lcinv)
    def rcin_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        rcinp = copy.deepcopy(self.desc[locx][locy]['rcin_path'])
        return(rcinp)
    def rcin(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = pl.__len__() - 2
        locx,locy = tuple(self.path2loc(pl))
        rcinp = copy.deepcopy(self.desc[locx][locy]['rcin_path'])
        rcinv = get_via_pl(self.list,rcinp) 
        return(rcinv)
    def sib_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        return(sibps)
    def sibs(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        sibvs = array_map(sibps,get_via_pl2,self.list)
        return(sibvs)
    def preceding_sib_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        try:
            seq = sibps.index(pl)
        except:
            return(sibps)
        else:
            return(sibps[:seq])
    def preceding_sibs(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        try:
            seq = sibps.index(pl)
        except:
            pre = sibps
        else:
            pre = sibps[:seq]
        sibvs = array_map(pre,get_via_pl2,self.list)
        return(sibvs)
    def following_sib_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        try:
            seq = sibps.index(pl)
        except:
            follow = sibps
        else:
            follow = sibps[(seq+1):]
        return(follow)
    def following_sibs(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        try:
            seq = sibps.index(pl)
        except:
            follow = sibps
        else:
            follow = sibps[(seq+1):]
        sibvs = array_map(follow,get_via_pl2,self.list)
        return(sibvs)
    def some_sib_paths(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        #here "some" mean "seqs"
        some = kwargs['some']
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        seq = self.desc[locx][locy]['sib_seq']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        #sibps = select_some(sibps,some)
        sibps = select_seqs(sibps,some)
        return(sibps)
    def some_sibs(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        #here some mean seqs
        some = kwargs['some']
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        seq = self.desc[locx][locy]['sib_seq']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        #sibps = select_some(sibps,some)
        sibps = select_seqs(sibps,some)
        sibvs = array_map(sibps,get_via_pl2,self.list)
        return(sibvs)
    def which_sib_path(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        which = kwargs['which']
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        seq = self.desc[locx][locy]['sib_seq']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        sibp = sibps[which]
        return(sibp)
    def which_sib(self,*sibseqs,**kwargs):
        if('pathlist' in kwargs):
            pl = kwargs['pathlist']
        else:
            pl = list(sibseqs)
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
        else:
            leaf_only = False
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
        else:
            non_leaf_only = False
        which = kwargs['which']
        locx,locy = tuple(self.path2loc(pl))
        ppl = self.desc[locx][locy]['parent_path']
        seq = self.desc[locx][locy]['sib_seq']
        sibps = self.son_paths(pathlist=ppl,leaf_only=leaf_only,non_leaf_only=non_leaf_only)
        sibp = sibps[which]
        sibv = get_via_pl(self.list,sibp)
        return(sibv)
    def search(self,value,**kwargs):
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
            prompt = 'leaf_only'
        else:
            leaf_only = False
            prompt = ''
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
            prompt = 'non_leaf_only'
        else:
            non_leaf_only = False
            prompt = ''
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = self.depth -1
        lpls = copy.deepcopy(self.desc[0][0]['leaf_descendant_paths'])
        nlpls = copy.deepcopy(self.desc[0][0]['non_leaf_descendant_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        nrslt = []
        for i in range(0,rslt.__len__()):
            pl = rslt[i]
            length = pl.__len__()
            cond1 = (length >= from_lv)
            cond2 = (length <= to_lv)
            v = get_via_pl(self.list,pl)
            cond3 = (v == value)
            if(cond1 & cond2 & cond3):
                nrslt.append(pl)
            else:
                pass
        showl = array_map(nrslt,pl_to_bracket_str)
        nrslt,showl = bat_sort(nrslt,nrslt,showl)
        if(type(value)==type("")):
            vstr = '"' + str(value) + '"'
        else:
            vstr = str(value)
        self.showlog = ['search '+ vstr + ' -'+prompt+' :']
        self.showlog.extend(showl)
        for_eachv(showl,print)
        return(nrslt)
    def cond_search(self,**kwargs):
        ###
        cond_func = kwargs['cond_func']
        if('cond_func_args' in kwargs):
            cond_func_args = kwargs['cond_func_args']
        else:
            cond_func_args = []
        ###
        if('leaf_only' in kwargs):
            leaf_only = kwargs['leaf_only']
            prompt = 'leaf_only'
        else:
            leaf_only = False
            prompt = ''
        if('non_leaf_only' in kwargs):
            non_leaf_only = kwargs['non_leaf_only']
            prompt = 'non_leaf_only'
        else:
            non_leaf_only = False
            prompt = ''
        if('from_lv' in kwargs):
            from_lv = kwargs['from_lv']
        else:
            from_lv = 1
        if('to_lv' in kwargs):
            to_lv = kwargs['to_lv']
        else:
            to_lv = self.depth -1
        lpls = copy.deepcopy(self.desc[0][0]['leaf_descendant_paths'])
        nlpls = copy.deepcopy(self.desc[0][0]['non_leaf_descendant_paths'])
        if(leaf_only):
            rslt = lpls
        elif(non_leaf_only):
            rslt = nlpls
        else:
            rslt = lpls+nlpls
        nrslt = []
        nvs = []
        for i in range(0,rslt.__len__()):
            pl = rslt[i]
            length = pl.__len__()
            cond1 = (length >= from_lv)
            cond2 = (length <= to_lv)
            v = get_via_pl(self.list,pl)
            cond3 = cond_func(v,pl,*cond_func_args)
            if(cond1 & cond2 & cond3):
                nrslt.append(pl)
                nvs.append(v)
            else:
                pass
        def showlog_append(ele1,ele2,*args):
            return(ele1 + ' : ' + str(ele2))
        showl = array_map(nrslt,pl_to_bracket_str)
        showl2 = array_map2(showl,nvs,map_func=showlog_append)
        nrslt,showl = bat_sort(nrslt,nrslt,showl)
        func_name = cond_func.__name__
        vstr = 'ele_value,ele_pathlist,' +str(cond_func_args)[1:-1]
        vstr = func_name + '(' + vstr + ')'
        self.showlog = ['search '+ vstr + ' -'+prompt+' :']
        self.showlog.extend(showl2)
        for_eachv(showl,print)
        return(nrslt)





from .cmmn import null,undefined


def find_fst_iv(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            return({'index':i,'value':ol[i]})
        else:
            pass
    return(null)


def find_fst_v(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            return(ol[i])
        else:
            pass
    return(null)


def find_fst_i(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            return(i)
        else:
            pass
    return(null)



def find_fst_not_iv(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return({'index':i,'value':ol[i]})
        else:
            pass
    return(null)


def find_fst_not_v(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return(ol[i])
        else:
            pass
    return(null)


def find_fst_not_i(ol,test_func,*args):
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return(i)
        else:
            pass
    return(null)



def find_lst_iv(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(cond):
            return({'index':i,'value':ol[i]})
        else:
            pass
    return(null)



def find_lst_i(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(cond):
            return(i)
        else:
            pass
    return(null)



def find_lst_v(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(cond):
            return(ol[i])
        else:
            pass
    return(null)


def find_lst_not_iv(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return({'index':i,'value':ol[i]})
        else:
            pass
    return(null)



def find_lst_not_i(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return(i)
        else:
            pass
    return(null)



def find_lst_not_v(ol,test_func,*args):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        cond = test_func(ol[i],*args)
        if(not(cond)):
            return(ol[i])
        else:
            pass
    return(null)


###
def find_which_iv(ol,test_func,which,*args):
    length = ol.__len__()
    seq = -1
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            seq = seq + 1
            if(seq == which):
                return({'index':i,'value':ol[i]})
            else:
                pass
        else:
            pass
    return(null)


def find_which_i(ol,test_func,which,*args):
    rslt = find_which_iv(ol,test_func,which,*args)
    rslt = rslt if(rslt == null) else rslt['index']
    return(rslt)

def find_which_v(ol,test_func,which,*args):
    rslt = find_which_iv(ol,test_func,which,*args)
    rslt = rslt if(rslt == null) else rslt['value']
    return(rslt)



def find_which_not_iv(ol,test_func,which,*args):
    length = ol.__len__()
    seq = -1
    for i in range(0,length):
        cond = not(test_func(ol[i],*args))
        if(cond):
            seq = seq + 1
            if(seq == which):
                return({'index':i,'value':ol[i]})
            else:
                pass
        else:
            pass
    return(null)



def find_which_not_i(ol,test_func,which,*args):
    rslt = find_which_not_iv(ol,test_func,which,*args)
    rslt = rslt if(rslt == null) else rslt['index']
    return(rslt)

def find_which_not_v(ol,test_func,which,*args):
    rslt = find_which_not_iv(ol,test_func,which,*args)
    rslt = rslt if(rslt == null) else rslt['value']
    return(rslt)


def find_some_iv(ol,test_func,*seqs,**kwargs):
    rslt =[]
    seqs = list(seqs)
    length = ol.__len__()
    seq = -1
    other_args = kwargs['other_args'] if('other_args' in kwargs) else []
    for i in range(0,length):
        cond = test_func(ol[i],*other_args)
        if(cond):
            seq = seq + 1
            if(seq in seqs):
                rslt.append({'index':i,'value':ol[i]})
            else:
                pass
        else:
            pass
    return(rslt)


def find_some_i(ol,test_func,*seqs,**kwargs):
    rslt = find_some_iv(ol,test_func,*seqs,**kwargs)
    rslt = list(map(lambda r:r['index'],rslt))
    return(rslt)


def find_some_v(ol,test_func,*seqs,**kwargs):
    rslt = find_some_iv(ol,test_func,*seqs,**kwargs)
    rslt = list(map(lambda r:r['value'],rslt))
    return(rslt)



def find_some_not_iv(ol,test_func,*seqs,**kwargs):
    rslt =[]
    seqs = list(seqs)
    length = ol.__len__()
    seq = -1
    other_args = kwargs['other_args'] if('other_args' in kwargs) else []
    for i in range(0,length):
        cond = not(test_func(ol[i],*other_args))
        if(cond):
            seq = seq + 1
            if(seq in seqs):
                rslt.append({'index':i,'value':ol[i]})
            else:
                pass
        else:
            pass
    return(rslt)


def find_some_not_i(ol,test_func,*seqs,**kwargs):
    rslt = find_some_not_iv(ol,test_func,*seqs,**kwargs)
    rslt = list(map(lambda r:r['index'],rslt))
    return(rslt)


def find_some_not_v(ol,test_func,*seqs,**kwargs):
    rslt = find_some_not_iv(ol,test_func,*seqs,**kwargs)
    rslt = list(map(lambda r:r['value'],rslt))
    return(rslt)



def find_all_iv(ol,test_func,*args):
    rslt =[]
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            rslt.append({'index':i,'value':ol[i]})
        else:
            pass
    return(rslt)


def find_all_i(ol,test_func,*args):
    rslt = find_all_iv(ol,test_func,*args)
    rslt = list(map(lambda r:r['index'],rslt))
    return(rslt)


def find_all_v(ol,test_func,*args):
    rslt = find_all_iv(ol,test_func,*args)
    rslt = list(map(lambda r:r['value'],rslt))
    return(rslt)



def find_all_not_iv(ol,test_func,*args):
    rslt =[]
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            pass
        else:
            rslt.append({'index':i,'value':ol[i]})
    return(rslt)



def find_all_not_i(ol,test_func,*args):
    rslt = find_all_not_iv(ol,test_func,*args)
    rslt = list(map(lambda r:r['index'],rslt))
    return(rslt)


def find_all_not_v(ol,test_func,*args):
    rslt = find_all_not_iv(ol,test_func,*args)
    rslt = list(map(lambda r:r['value'],rslt))
    return(rslt)


############

def find_fst_gt_iv(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_fst_iv(ol,test_func,value)
    return(rslt)


def find_fst_gt_i(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_fst_i(ol,test_func,value)
    return(rslt)


def find_fst_gt_v(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_fst_v(ol,test_func,value)
    return(rslt)


def find_lst_gt_iv(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_lst_iv(ol,test_func,value)
    return(rslt)


def find_lst_gt_i(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_lst_i(ol,test_func,value)
    return(rslt)


def find_lst_gt_v(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_lst_v(ol,test_func,value)
    return(rslt)




def find_which_gt_iv(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_which_iv(ol,test_func,which,value)
    return(rslt)


def find_which_gt_i(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_which_i(ol,test_func,which,value)
    return(rslt)


def find_which_gt_v(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_which_v(ol,test_func,which,value)
    return(rslt)



def find_some_gt_iv(ol,value,*seqs):
    test_func = lambda r,v:(r>v)
    rslt = find_some_iv(ol,test_func,value,*seqs)
    return(rslt)


def find_some_gt_i(ol,value,*seqs):
    test_func = lambda r,v:(r>v)
    rslt = find_some_i(ol,test_func,value,*seqs)
    return(rslt)


def find_some_gt_v(ol,value,*seqs):
    test_func = lambda r,v:(r>v)
    rslt = find_some_v(ol,test_func,value,*seqs)
    return(rslt)



def find_all_gt_iv(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_all_iv(ol,test_func,value)
    return(rslt)


def find_all_gt_i(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_all_i(ol,test_func,value)
    return(rslt)


def find_all_gt_v(ol,value):
    test_func = lambda r,v:(r>v)
    rslt = find_all_v(ol,test_func,value)
    return(rslt)



def find_fst_lt_iv(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_fst_iv(ol,test_func,value)
    return(rslt)


def find_fst_lt_i(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_fst_i(ol,test_func,value)
    return(rslt)


def find_fst_lt_v(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_fst_v(ol,test_func,value)
    return(rslt)


def find_lst_lt_iv(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_lst_iv(ol,test_func,value)
    return(rslt)


def find_lst_lt_i(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_lst_i(ol,test_func,value)
    return(rslt)


def find_lst_lt_v(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_lst_v(ol,test_func,value)
    return(rslt)


def find_which_lt_iv(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_which_iv(ol,test_func,which,value)
    return(rslt)


def find_which_lt_i(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_which_i(ol,test_func,which,value)
    return(rslt)


def find_which_lt_v(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_which_v(ol,test_func,which,value)
    return(rslt)

def find_some_lt_iv(ol,value,*seqs):
    test_func = lambda r,v:(r<v)
    rslt = find_some_iv(ol,test_func,value,*seqs)
    return(rslt)


def find_some_lt_i(ol,value,*seqs):
    test_func = lambda r,v:(r<v)
    rslt = find_some_i(ol,test_func,value,*seqs)
    return(rslt)


def find_some_lt_v(ol,value,*seqs):
    test_func = lambda r,v:(r<v)
    rslt = find_some_v(ol,test_func,value,*seqs)
    return(rslt)


def find_all_lt_iv(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_all_iv(ol,test_func,value)
    return(rslt)


def find_all_lt_i(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_all_i(ol,test_func,value)
    return(rslt)


def find_all_lt_v(ol,value):
    test_func = lambda r,v:(r<v)
    rslt = find_all_v(ol,test_func,value)
    return(rslt)




def find_fst_eq_iv(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_fst_iv(ol,test_func,value)
    return(rseq)


def find_fst_eq_i(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_fst_i(ol,test_func,value)
    return(rseq)


def find_fst_eq_v(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_fst_v(ol,test_func,value)
    return(rseq)


def find_lst_eq_iv(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_lst_iv(ol,test_func,value)
    return(rseq)


def find_lst_eq_i(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_lst_i(ol,test_func,value)
    return(rseq)


def find_lst_eq_v(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_lst_v(ol,test_func,value)
    return(rseq)


def find_which_eq_iv(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_which_iv(ol,test_func,which,value)
    return(rseq)


def find_which_eq_i(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_which_i(ol,test_func,which,value)
    return(rseq)


def find_which_eq_v(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_which_v(ol,test_func,which,value)
    return(rseq)

def find_some_eq_iv(ol,value,*seqs):
    test_func = lambda r,v:(r==v)
    rseq = find_some_iv(ol,test_func,value,*seqs)
    return(rseq)


def find_some_eq_i(ol,value,*seqs):
    test_func = lambda r,v:(r==v)
    rseq = find_some_i(ol,test_func,value,*seqs)
    return(rseq)


def find_some_eq_v(ol,value,*seqs):
    test_func = lambda r,v:(r==v)
    rseq = find_some_v(ol,test_func,value,*seqs)
    return(rseq)


def find_all_eq_iv(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_all_iv(ol,test_func,value)
    return(rseq)


def find_all_eq_i(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_all_i(ol,test_func,value)
    return(rseq)


def find_all_eq_v(ol,value):
    test_func = lambda r,v:(r==v)
    rseq = find_all_v(ol,test_func,value)
    return(rseq)






import random
import copy

def rand_some_indexes(si,ei,n,**kwargs):
    if('sort' in kwargs):
        sort = kwargs['sort']
    else:
        sort = True
    rslt = []
    arr = init_range(si,ei,1)
    c = 0
    while(c<n):
        r = random.randrange(0,ei)
        p = arr.pop(r)
        rslt.append(p)
        ei = ei - 1
        c = c + 1
    if(sort):
        rslt.sort()
    else:
        pass
    return(rslt)


def rand_sub(arr,*args,**kwargs):
    arr = copy.deepcopy(arr)
    lngth = arr.__len__()
    args = list(args)
    if(args.__len__() == 0):
        n = random.randrange(0,lngth)
    else:
        n = args[0]
        if(n>lngth):
            n = lngth
        else:
            pass
    indexes = rand_some_indexes(0,lngth,n,**kwargs)
    narr = some_keeping_order(arr,*indexes)
    return(narr)

